sobol<-function(modele,dim_x,dimnames_x,methode=1,x=NULL,
                logx=rep(FALSE,dim_x),logy=FALSE,S=10000,R=1,Sij=TRUE,
                n.trees=25000,lois=rep(0,dim_x),
                paramlois=array(0,dim=c(2,dim_x))){
#---------------------------------------------------------------------------|
#Fonction pour calculer les indices de sensibilite de Sobol                 |
#---------------------------------------------------------------------------|
#Arguments                                                                  |
#obligatoires : modele : objet contenant le modele                          |
#               dim_x : nombre de parametres d'entree dans le modele        |
#               dimnames_x : noms de parametres d'entree dans le modele     |
#               (defini comme : c("V1","V2","V3",...))                      |
#                                                                           |
#Argument                                                                   |
#facultatif :                                                               |
#             methode : numero de la methode                                |
#             x : matrice de variables d'entree (pour calculer les bornes   |
#                de la loi uniforme si necessaire)                          |
#             logx : vecteur logx=c(TRUE,TRUE,...,TRUE), correspondant aux  |
#                parametres d'entree transformees en log lors de            |
#                l'ajustement du modele                                     |
#             logy : mettre logy=TRUE si la variable de sortie a ete        |
#                transformee en log lors de l'ajustement du modele          |
#             S : nombre de simulations                                     |
#             R : nombre de repetitions du programme                        |
#             Sij : TRUE pour calculer les indices d'ordre 2                |
#             lois : vecteur contenant les types de distribution de proba   |
#               pour chaque Vi                                              |
#               0=uniforme ; 1=normale ; 2=lognormale ; 3=weibull           |
#               Par defaut, on prend la loi uniforme                        |
#             paramlois : tableau contenant les 2 parametres de chaque loi  |
#               pour chaque Vi (range par colonne) :                        |
#               (min,max) pour uniforme, (moy,ecart-type) pour loi normale, |
#               (moy du log, ecart-type du log) pour loi lognormale,        |
#               (forme,echelle) pour loi de Weibull                         |
#               Par defaut, on a la loi uniforme avec (min,max) calcules a  |
#               partir des parametres d'entree x                            |
#             n.trees : nombre d'arbres dans le cas du BOOSTING             |
#---------------------------------------------------------------------------|
#Sortie : les indices Si, Sij et STi a la fin du fichier "res-SOBOL.out"    |
#         (fichier non cras)                                              |
#---------------------------------------------------------------------------|
		
#Initialisation des vecteurs contenant les indices de sensibilite

S1<-array(0,dim=c(R,dim_x))
S2<-array(0,dim=c(R,dim_x,dim_x))
St<-array(0,dim=c(R,dim_x))

#--------
#Boucle |
#--------

for(k in 1:R){
	      
#Tirage des echantillons
  X1<-matrix(0,nr=S,nc=dim_x,dimnames=list(1:S,dimnames_x))
  X2<-matrix(0,nr=S,nc=dim_x,dimnames=list(1:S,dimnames_x))

  for(i in 1:dim_x){

    if (lois[i]==0){ # loi uniforme
      if (paramlois[1,i]==0 & paramlois[2,i]==0){
        if (logx[i]==FALSE){
          X1[,i]<-runif(S,min=min(x[,i]),max=max(x[,i]))
          X2[,i]<-runif(S,min=min(x[,i]),max=max(x[,i]))}
        else{
          X1[,i]<-runif(S,min=exp(min(x[,i])),max=exp(max(x[,i])))
          X2[,i]<-runif(S,min=exp(min(x[,i])),max=exp(max(x[,i])))}}

      else{
        X1[,i]<-runif(S,min=paramlois[1,i],max=paramlois[2,i])
        X2[,i]<-runif(S,min=paramlois[1,i],max=paramlois[2,i])}}

    if (lois[i]==1){ # loi normale
      X1[,i]<-rnorm(S,mean=paramlois[1,i],sd=paramlois[2,i])
      X2[,i]<-rnorm(S,mean=paramlois[1,i],sd=paramlois[2,i])}

    if (lois[i]==2){ # loi lognormale
      X1[,i]<-rlnorm(S,meanlog=paramlois[1,i],sdlog=paramlois[2,i])
      X2[,i]<-rlnorm(S,meanlog=paramlois[1,i],sdlog=paramlois[2,i])}

    if (lois[i]==3){ # loi weibull
      X1[,i]<-rweibull(S,shape=paramlois[1,i],scale=paramlois[2,i])
      X2[,i]<-rweibull(S,shape=paramlois[1,i],scale=paramlois[2,i])}

    if (logx[i]==TRUE){ X1[,i]=log(X1[,i]); X2[,i]=log(X2[,i])}
  }

# Modifications Elena pour variables correlees
  X1[,11]=X1[,8]*runif(S,min=0.01,max=0.1)
  X2[,11]=X2[,8]*runif(S,min=0.01,max=0.1)
  X1[,12]=X1[,9]*runif(S,min=0.01,max=0.1)
  X2[,12]=X2[,9]*runif(S,min=0.01,max=0.1)
  X1[,13]=X1[,10]*runif(S,min=0.01,max=0.1)
  X2[,13]=X2[,10]*runif(S,min=0.01,max=0.1)

#---------------------------
#Calcul de la moyenne de y |
#---------------------------
  if (methode==1){ 
    pred<-predict.glm(modele,data.frame(X1))
    lien<-modele$family[2] # fct lien differente de l'identite
    if(lien=="inverse"){pred<-1/pred}
    else{ if(lien=="log"){pred<-exp(pred)}
          else{ if(lien=="1/mu^2"){pred<-1/sqrt(pred)}}}}

  if (methode==2){ pred<-predict.mvr(modele,as.matrix(X1),nlv=dim_x)}
  if (methode==3){ pred<-predict(modele,X1)}
  if (methode==4){ pred<-predict.gbm(modele,X1,n.trees=n.trees)}
  if (methode==5){ pred<-predict(modele,X1)}
  if (methode==6){ pred<-predict(modele,X1)}

  if (logy==TRUE){ pred=exp(pred)}

  my<-(1/S)*sum(pred)

#-------------
#Calcul de V0 |
#-------------
  V0<-((1/S)*sum(pred^2))-my^2

#---------------
#Calcul des Vi |
#---------------
  V<-rep(0,dim_x)

  for(i in 1:dim_x){

    pred1<-pred

    X2b<-X2 
    X2b[,i]<-X1[,i]
    if (i>=8 & i<=10){ X2b[,i+3]<-X1[,i+3]} # modifs pour Elena

    if (methode==1){ 
      pred2<-predict.glm(modele,data.frame(X2b))
      lien<-modele$family[2] # fct lien differente de l'identite
      if(lien=="inverse"){pred2<-1/pred2}
      else{ if(lien=="log"){pred2<-exp(pred2)}
            else{ if(lien=="1/mu^2"){pred2<-1/sqrt(pred2)}}}}
    if (methode==2){ pred2<-predict.mvr(modele,as.matrix(X2b),nlv=dim_x)}
    if (methode==3){ pred2<-predict(modele,X2b)}
    if (methode==4){ pred2<-predict.gbm(modele,X2b,n.trees=n.trees)}
    if (methode==5){ pred2<-predict(modele,X2b)}
    if (methode==6){ pred2<-predict(modele,X2b)}

    if (logy==TRUE){ pred1=exp(pred1); pred2=exp(pred2)}

    V[i]<-((1/S)*sum(pred1*pred2))-my^2}

#----------------
#Calcul des Vij |
#----------------
  V2<-matrix(0,nr=dim_x,nc=dim_x)

  if (Sij==TRUE){
    for(i in 1:(dim_x-1)){
      for(j in (i+1):dim_x){

        pred1<-pred

        X2b<-X2
        X2b[,i]<-X1[,i]
        X2b[,j]<-X1[,j]

        if (methode==1){ 
          pred2<-predict.glm(modele,data.frame(X2b))
          lien<-modele$family[2] # fct lien differente de l'identite
          if(lien=="inverse"){pred2<-1/pred2}
          else{ if(lien=="log"){pred2<-exp(pred2)}
                else{ if(lien=="1/mu^2"){pred2<-1/sqrt(pred2)}}}}
        if (methode==2){ pred2<-predict.mvr(modele,as.matrix(X2b),nlv=dim_x)}
        if (methode==3){ pred2<-predict(modele,X2b)}
        if (methode==4){ pred2<-predict.gbm(modele,X2b,n.trees=n.trees)}
        if (methode==5){ pred2<-predict(modele,X2b)}
        if (methode==6){ pred2<-predict(modele,X2b)}

        if (logy==TRUE){ pred1=exp(pred1); pred2=exp(pred2)}

        V2[i,j]<-((1/S)*sum(pred1*pred2))-V[i]-V[j]-my^2}}}

#----------------
#Calcul des Vti |
#----------------
  Vt<-rep(0,dim_x)

  for(i in 1:dim_x){
    
    pred1<-pred

    X1b<-X1
    X1b[,i]<-X2[,i]
    if (i>=8 & i<=10){ X1b[,i+3]<-X2[,i+3]} # modifs pour Elena

    if (methode==1){ 
      pred2<-predict.glm(modele,data.frame(X1b))
      lien<-modele$family[2] # fct lien differente de l'identite
      if(lien=="inverse"){pred2<-1/pred2}
      else{ if(lien=="log"){pred2<-exp(pred2)}
            else{ if(lien=="1/mu^2"){pred2<-1/sqrt(pred2)}}}}
    if (methode==2){ pred2<-predict.mvr(modele,as.matrix(X1b),nlv=dim_x)}
    if (methode==3){ pred2<-predict(modele,X1b)}
    if (methode==4){ pred2<-predict.gbm(modele,X1b,n.trees=n.trees)}
    if (methode==5){ pred2<-predict(modele,X1b)}
    if (methode==6){ pred2<-predict(modele,X1b)}

    if (logy==TRUE){ pred1=exp(pred1); pred2=exp(pred2)}

    Vt[i]<-((1/S)*sum(pred1*pred2))-my^2}

#-----------------------------
#Calcul des indices de Sobol |
#-----------------------------
  S1[k,]<-V/V0
  S2[k,,]<-V2/V0
  St[k,]<-1-(Vt/V0)}

#------------------
#Fin de la boucle |
#------------------

#Calcul des moyennes, min, max et ecart-types

#Initialisation des vecteurs
mS1<-rep(0,dim_x)
mS2<-matrix(0,nr=dim_x,nc=dim_x)
mSt<-rep(0,dim_x)
minS1<-rep(0,dim_x)
minS2<-matrix(0,nr=dim_x,nc=dim_x)
minSt<-rep(0,dim_x)
maxS1<-rep(0,dim_x)
maxS2<-matrix(0,nr=dim_x,nc=dim_x)
maxSt<-rep(0,dim_x)
sigS1<-rep(0,dim_x)
sigS2<-matrix(0,nr=dim_x,nc=dim_x)
sigSt<-rep(0,dim_x)

for(i in 1:dim_x){
  mS1[i]<-mean(S1[,i])
  minS1[i]<-min(S1[,i])
  maxS1[i]<-max(S1[,i])
  sigS1[i]<-sqrt(var(S1[,i]))
}

for(i in 1:(dim_x-1)){
  for(j in (i+1):dim_x){
    mS2[i,j]<-mean(S2[,i,j])
    minS2[i]<-min(S2[,i,j])
    maxS2[i]<-max(S2[,i,j])
    sigS2[i]<-sqrt(var(S2[,i,j]))
  }
}

for(i in 1:dim_x){
  mSt[i]<-mean(St[,i])
  minSt[i]<-min(St[,i])
  maxSt[i]<-max(St[,i])
  sigSt[i]<-sqrt(var(St[,i]))
}

#-----------------------
#Affichage dans fichier |
#-----------------------

#Preparation des donnees pour l'affichage
mS1<-format(mS1,digits=3)
mS2<-format(mS2,digits=3)
mSt<-format(mSt,digits=3)
minS1<-format(minS1,digits=3)
minS2<-format(minS2,digits=3)
minSt<-format(minSt,digits=3)
maxS1<-format(maxS1,digits=3)
maxS2<-format(maxS2,digits=3)
maxSt<-format(maxSt,digits=3)
sigS1<-format(sigS1,digits=3)
sigS2<-format(sigS2,digits=3)
sigSt<-format(sigSt,digits=3)

FSOB <- file("res-SOBOL.out","a+")
write("--------------------------------------------------",file=FSOB)

if (methode==1){
  write("Modele GLM : Indices de sensibilite de Sobol",file=FSOB)
  print("Sobol GLM termine");}
if (methode==2){
  write("Modele PLS : Indices de sensibilite de Sobol",file=FSOB)
  print("Sobol PLS termine");}
if (methode==3){
  write("Modele SVM : Indices de sensibilite de Sobol",file=FSOB)
  print("Sobol SVM termine");}
if (methode==4){
  write("Modele BOOSTING : Indices de sensibilite de Sobol",file=FSOB)
  print("Sobol BOOSTING termine");}
if (methode==5){
  write("Modele FORETS ALEATOIRES : Indices de sensibilite de Sobol",file=FSOB)
  print("Sobol FORETS ALEATOIRES termine");}
if (methode==6){
  write("Modele RESEAU de NEURONES : Indices de sensibilite de Sobol",file=FSOB)
  print("Sobol RESEAU de NEURONES termine");}

write(" ",file=FSOB)

write("------------------",file=FSOB)
write("| Les indices Si |",file=FSOB)
write("------------------",file=FSOB)

for(i in 1:dim_x){ write(c("S",i," : Moy=",mS1[i]," Min=",minS1[i]," Max=",maxS1[i]," E-t=",sigS1[i]),ncolumns=10,file=FSOB)}
write(" ",file=FSOB)

if (Sij==TRUE){
  write("-------------------",file=FSOB)
  write("| Les indices Sij |",file=FSOB)
  write("-------------------",file=FSOB)

  for(i in 1:(dim_x-1)){
    for(j in (i+1):dim_x){
      write(c("S",i,j," : Moy=",mS2[i,j]," Min=",minS2[i]," Max=",maxS2[i]," E-t=",sigS2[i]),ncolumns=11,file=FSOB)}}
  write(" ",file=FSOB)}

write("-------------------",file=FSOB)
write("| Les indices Sti |",file=FSOB)
write("-------------------",file=FSOB)

for(i in 1:dim_x){ write(c("St",i," : Moy=",mSt[i]," Min=",minSt[i]," Max=",maxSt[i]," E-t=",sigSt[i]),ncolumns=10,file=FSOB)}
write(" ",file=FSOB)

close(FSOB)
}
