exploreSVM<-function(xa,ya,xt,yt,logy=FALSE,save=TRUE,critere="R2",cmin=1,cmax=101,cpas=50,epsmin=0.01,epsmax=0.51,epspas=0.25,gmin=0.5,gmax=10.5,gpas=5){
    
#-----------------------------------------------------------------------------
# Fonction explorant les valeurs des parametres dans	    		     |
# le cas d'un SVM a kernel radial (=gaussien).		   		     |
#-----------------------------------------------------------------------------
# Arguments obligatoires :    xa,ya : La base d'apprentissage.               |
#		              xt,yt : La base de test.                       |
# Arguments facultatifs :                                                    |
#               logy : si logy=TRUE, il faut passer en exp(y) avant d'evaluer|
#                         les criteres                                       |
#               save : est-ce qu'on sauve les criteres dans un fichier ?     |
#               critere : Quel critere est privilegie lors du retour des     |
#                         valeurs optimales (="R2","R2ajuste","MSE","NMSE",  |
#                         "SMSE","SSD","B","BN","RM","MRR","SRR").           |
#		cmin,cmax, cpas : La plage d'exploration du cout.            |
#		epsmin,epsmax,epspas : La plage d'exploration de la marge.   |
#               gmin,gmax,gpas: La plage d'exploration de la portee.         |
#-----------------------------------------------------------------------------
# Sorties: La fonction enregistre les criteres (cf. criteres.R)              |
#          (r2m,r2am,msem,nmsem,smsem,ssdm,bm,bnm,rmm,mrrm,srrm) dans        |
#          le fichier res-exploreSVM.Rdata, elle renvoie galement les       | 
#          valeurs optimales trouvees ainsi que les valeurs des parametres   |
#          correspondants.                                                   |
#----------------------------------------------------------------------------|
#      Auteur : B. Iooss                                                     |
#----------------------------------------------------------------------------|  

# Creation des vecteurs des parametres

  C<-seq(cmin,cmax,by=cpas)	
  eps<-seq(epsmin,epsmax,by=epspas)	
  g<-seq(gmin,gmax,by=gpas)
 	
# Creation des variables destinees a stocker les criteres et parametres optimaux
# Nom: "sigleducritere"+"min/max" selon le cas et "sigleducritere"+"ijk" pour 
# indiquer les coordonnees correspondantes.

  r2max<-(-Inf);rijk<-c(0,0,0);
  r2amax<-(-Inf);r2aijk<-rijk;
  msemin<-Inf;mijk<-rijk;
  nmsemin<-Inf;nijk<-rijk;
  smsemin<-Inf;smijk<-rijk;
  ssdmin<-Inf;ssijk<-rijk;
  bmin<-Inf;bijk=rijk;
  bnmin<-Inf;bnijk=rijk;
  rmmin=Inf;rmijk=rijk;
  mrrmin=Inf;mrrijk=rijk;
  srrmin=Inf;srrijk=rijk;
  ijk=rijk;

# Creation des tableaux destines a contenir les valeurs des criteres

  r2m<-array(0,c(length(C),length(eps),length(g))); r2am<-r2m; msem<-r2m;
  nmsem<-r2m; smsem<-r2m; ssdm <-r2m; bm=r2m; bnm=r2m; rmm=r2m; mrrm=r2m; srrm=r2m; 

# Boucle principale

  for (i in C){
     for (j in eps){
        for (k in g){

	   # Creation et utilisation du modele SVM.
	   if (is.data.frame(xa)==TRUE) {
	      model=svm(ya~.,data=xa,kernel="radial",cost=i,epsilon=j,gamma=k);	
	   }
	   else {
              model<-svm(xa,ya,kernel="radial",cost=i,epsilon=j,gamma=k);
	   }
	   pred<-predict(model,xt);

	   # Calcul des index correspondant a l'iteration.	
	   indexi<-(i-cmin)/cpas+1;indexj<-(j-epsmin)/epspas+1;indexk<-(k-gmin)/gpas+1;
	   
	   # Stockage des criteres trouves et eventuellement mise a jour des optimums.		

	   if (logy==TRUE){ resultat=criteres(exp(yt),exp(pred),dim_x=dim(xa)[2])}
	   else{ resultat=criteres(yt,pred,dim_x=dim(xa)[2])}

	   r2m[indexi,indexj,indexk]<-resultat[1];
	   if(r2m[indexi,indexj,indexk]>r2max){r2max<-r2m[indexi,indexj,indexk];rijk<-c(i,j,k);};
	   r2am[indexi,indexj,indexk]<-resultat[2];
	   if(r2am[indexi,indexj,indexk]>r2amax){r2amax<-r2am[indexi,indexj,indexk];r2aijk<-c(i,j,k);};
           msem[indexi,indexj,indexk]<-resultat[3];
	   if(msem[indexi,indexj,indexk]<msemin){msemin<-msem[indexi,indexj,indexk];mijk<-c(i,j,k);};
           nmsem[indexi,indexj,indexk]<-resultat[4];
	   if(nmsem[indexi,indexj,indexk]<nmsemin){nmsemin<-nmsem[indexi,indexj,indexk];nijk<-c(i,j,k);};		
           smsem[indexi,indexj,indexk]<-resultat[5];
	   if(smsem[indexi,indexj,indexk]<smsemin){smsemin<-smsem[indexi,indexj,indexk];smijk<-c(i,j,k);};		
           ssdm[indexi,indexj,indexk]<-resultat[6];
	   if(ssdm[indexi,indexj,indexk]<ssdmin){ssdmin<-ssdm[indexi,indexj,indexk];ssdijk<-c(i,j,k);};
           bm[indexi,indexj,indexk]<-resultat[7];
	   if(bm[indexi,indexj,indexk]<bmin){bmin<-bm[indexi,indexj,indexk];bijk<-c(i,j,k);};
           bnm[indexi,indexj,indexk]<-resultat[8];
	   if(bnm[indexi,indexj,indexk]<bnmin){bnmin<-bnm[indexi,indexj,indexk];bnijk<-c(i,j,k);};
           rmm[indexi,indexj,indexk]<-resultat[9];
	   if(rmm[indexi,indexj,indexk]<rmmin){rmmin<-rmm[indexi,indexj,indexk];rmijk<-c(i,j,k);};
           mrrm[indexi,indexj,indexk]<-resultat[10];
	   if(mrrm[indexi,indexj,indexk]<mrrmin){mrrmin<-mrrm[indexi,indexj,indexk];mrrijk<-c(i,j,k);};
           srrm[indexi,indexj,indexk]<-resultat[11];
	   if(srrm[indexi,indexj,indexk]<srrmin){srrmin<-srrm[indexi,indexj,indexk];srrijk<-c(i,j,k);};
        };
     };
  };

# Ecriture des tableaux dans leurs fichiers respectifs.

   if(save==TRUE){ save(r2m,r2am,msem,nmsem,smsem,ssdm,bm,bnm,rmm,mrrm,srrm,file="res-exploreSVM.Rdata");}

   if (critere=="R2"){
      ijk=rijk;
   }
   else if (critere=="R2ajuste"){
      ijk=r2aijk;
   }
   else if (critere=="MSE"){
      ijk=mijk;
   }
   else if (critere=="NMSE"){
      ijk=nijk;
   }
   else if (critere=="SMSE"){
      ijk=smijk;
   }
   else if (critere=="SSD"){
      ijk=ssijk;
   }
   else if (critere=="B"){
      ijk=bijk;
   }
   else if (critere=="BN"){
      ijk=bnijk;
   }
   else if (critere=="RM"){
      ijk=rmijk;
   }
   else if (critere=="MRR"){
      ijk=mrrijk;
   }
   else if (critere=="SRR"){
      ijk=srrijk;
   }

   return(ijk);

}
