quantileconverg<-function(x,nom=c(NA),nrepet=1,nboot=100,fboot=4,kboot=1,
                       conf=0.95,moments=FALSE){

#-----------------------------------------------------------------------
#Etude de la representativite des donnees en analysant la convergence  |
#des quantiles a 75%, 90%, 95% et 99% d'une variable x,                |
#avec des intervalles de confiance donnes par bootstrap                |
#----------------------------------------------------------------------|
#Arguments :                                                           |
#                                                                      |
#obligatoire : x : variable d'entree                                   |
#                                                                      |
#facultatif : nom    : vecteur des noms de variable (7 caracteres max) |
#                     (par defaut : header de x)                       |
#             nrepet : nombre de repetitions de graphes de convergence |
#             nboot  : nombre de repliques bootstrap                   |
#             fboot  : 1ere taille d'echantillon pour la convergence   |
#             kboot  : increment de la taille d'echantillon            |
#             conf   : niveau de confiance pour l'intervalle bootstrap |
#             moments: TRUE pour avoir les quantiles a 95% et 99%      |
#                      en plus des quantiles a 50% et 75%              |
#----------------------------------------------------------------------|
#Sortie : Graphes de convergence                                       |
#----------------------------------------------------------------------|
#Utilisation :                                                         |
#  -Au prealable, il faut allouer x sous R                             |
#    Exemple : mat<-read.table(file="MOX_DEI.txt",header=TRUE,dec=",") |
#            x<-mat[,2:length(mat)]                                    |
#            x<-x+runif(dim(x)[[1]]*dim(x)[[2]],min=-0.0005,max=0.0005)|
#            x<-read.table(file="nom_fichier")                         |
#----------------------------------------------------------------------|
#      Auteur : B. Iooss                                               |
#----------------------------------------------------------------------|  
  
# *********************************************************************
# FONCTION UTILISEE :
#      bootstats.R (intervalle de confiance bootstrap)
# *********************************************************************
# LIBRAIRIES REQUISES
  library(boot)
  library(e1071) # A INSTALLER
  
# *********************************************************************
# INITIALISATIONS

# Transformation de x en matrice
  if (is.matrix(x)==FALSE & is.vector(x)==TRUE) x<-matrix(x,ncol=1)
  
# Cacul des dimensions de x
  n<-dim(x)[1]
  np<-dim(x)[2]

# Noms des variables
  for(i in 1:np){
    if (is.na(nom[i])==T){
      if (is.null(dimnames(x)[[2]][i])==F) nom[i]<-dimnames(x)[[2]][i]
      else nom[i] <- ""}}
#  x<-matrix(c(x),nr=n,nc=np,dimnames=list(1:n,nom[1:np]))

  xx <- matrix(0,nr=n,nc=np)
  
  moy <- matrix(0,nr=np,nc=5)
  moymean <- matrix(0,nr=n,nc=np)
  moybias <- matrix(0,nr=n,nc=np)
  moycimin <- matrix(0,nr=n,nc=np)
  moycimax <- matrix(0,nr=n,nc=np)
  ect <- matrix(0,nr=np,nc=5)
  ectmean <- matrix(0,nr=n,nc=np)
  ectbias <- matrix(0,nr=n,nc=np)
  ectcimin <- matrix(0,nr=n,nc=np)
  ectcimax <- matrix(0,nr=n,nc=np)

  if (moments==TRUE){
    skew <- matrix(0,nr=np,nc=5)
    skewmean <- matrix(0,nr=n,nc=np)
    skewbias <- matrix(0,nr=n,nc=np)
    skewcimin <- matrix(0,nr=n,nc=np)
    skewcimax <- matrix(0,nr=n,nc=np)
    kurt <- matrix(0,nr=np,nc=5)
    kurtmean <- matrix(0,nr=n,nc=np)
    kurtbias <- matrix(0,nr=n,nc=np)
    kurtcimin <- matrix(0,nr=n,nc=np)
    kurtcimax <- matrix(0,nr=n,nc=np)}
  
# Verification des arguments
  if (nboot <= 1) {
    print("nboot doit etre > 1, on prendra nboot=100")
    nboot <- 100}
  if (fboot >= n) {
    print("fboot doit etre < nb de donnees, on prendra fboot=1")
    fboot <- 1}
  if (np > 1){
    print("On ne peut pas calculer les quantiles quand on a plusieurs colonnes dans x")}
  
# *********************************************************************

# BOUCLE sur les REPETITIONS de l'ETUDE de la CONVERGENCE
  for (jj in 1:nrepet){
    
# Reechantillonnage de la base initiale
    for (j in 1:np) xx[,j] <- sample(x[,j],n)

# BOUCLE sur la TAILLE d'ECHANTILLON
    i <- fboot
    index <- NULL
    while (i <= n){

# Bootstrap sur le quantile a 75%
      b <- boot(xx[1:i,],function(s,j){quantile(as.data.frame(s)[j,],probs=0.75)},R=nboot)
      moy <- bootstats(b,conf,"basic")[,1:5]
      moymean[i,] <- moy[,1] # Moyenne echantillon original
      moybias[i,] <- moy[,2] + moy[,1] # estimateur bootstrap
      moycimin[i,] <- moy[,4] # Intervalle de confiance
      moycimax[i,] <- moy[,5]

# Bootstrap sur le quantile a 90%
      b <- boot(xx[1:i,],function(s,j){quantile(as.data.frame(s)[j,],probs=0.90)},R=nboot)
      ect <- bootstats(b,conf,"basic")[,1:5]
      ectmean[i,] <- ect[,1] # ecart-type echantillon original
      ectbias[i,] <- ect[,2] + ect[,1] # estimateur bootstrap
      ectcimin[i,] <- ect[,4] # Intervalle de confiance
      ectcimax[i,] <- ect[,5]

      if (moments==TRUE){
# Bootstrap sur le quantile a 95%
        b <- boot(xx[1:i,],function(s,j){quantile(as.data.frame(s)[j,],probs=0.95)},R=nboot)
        skew <- bootstats(b,conf,"basic")[,1:5]
        skewmean[i,] <- skew[,1] # Skewness echantillon original
        skewbias[i,] <- skew[,2] + skew[,1] # estimateur bootstrap
        skewcimin[i,] <- skew[,4] # Intervalle de confiance
        skewcimax[i,] <- skew[,5]

# Bootstrap sur le quantile a 99%
        b <- boot(xx[1:i,],function(s,j){quantile(as.data.frame(s)[j,],probs=0.99)},R=nboot)
        kurt <- bootstats(b,conf,"basic")[,1:5]
        kurtmean[i,] <- kurt[,1] # Kurtosis echantillon original
        kurtbias[i,] <- kurt[,2] + kurt[,1] # estimateur bootstrap
        kurtcimin[i,] <- kurt[,4] # Intervalle de confiance
        kurtcimax[i,] <- kurt[,5]
      }

      index <- c(index,i)
      i <- i+kboot
    }

# *********************************************************************
# GRAPHIQUES

    for (j in 1:np){
      x11()
      if (moments==FALSE) op <- par(mfcol=c(2,1))
      else op <- par(mfcol=c(2,2))
      
      par(mfg=c(1,1))
      plot(index,moymean[index,j],pch=20,xlab="",ylab="Quantile 75%",ylim=c(min(moycimin[index,j]),max(moycimax[index,j])),main="")
      lines(index,moybias[index,j])
      lines(index,moycimin[index,j])
      lines(index,moycimax[index,j])
      if (moments==FALSE) title(c("Convergence des statistiques",nom[j]),sub="point : estimateur echantillon\n ligne : estimateur bootstrap + IC a 95%")
      else title("Convergence des statistiques",sub="point : estimateur echantillon")
      par(mfg=c(2,1))
      plot(index,ectmean[index,j],pch=20,xlab="taille de l'echantillon",ylab="Quantile 90%",ylim=c(min(ectcimin[index,j]),max(ectcimax[index,j])))
      lines(index,ectbias[index,j])
      lines(index,ectcimin[index,j])
      lines(index,ectcimax[index,j])
    
      if (moments==TRUE){
        par(mfg=c(1,2))
        plot(index,skewmean[index,j],pch=20,xlab="",ylab="Quantile 95%",ylim=c(min(skewcimin[index,j]),max(skewcimax[index,j])),main="")
        lines(index,skewbias[index,j])
        lines(index,skewcimin[index,j])
        lines(index,skewcimax[index,j])
        title(nom[j],sub="ligne : estimateur bootstrap + IC a 95%")
        par(mfg=c(2,2))
        plot(index,kurtmean[index,j],pch=20,xlab="taille de l'echantillon",ylab="Quantile 99%",ylim=c(min(kurtcimin[index,j]),max(kurtcimax[index,j])))
        lines(index,kurtbias[index,j])
        lines(index,kurtcimin[index,j])
        lines(index,kurtcimax[index,j])
      }
    }
  }
}
