samplingLHScontraint<-function(dim_x,nom=c(NA),N=1,lois=rep(0,dim_x),
                  paramlois=array(0,dim=c(4,dim_x)),contraint=rep(0,dim_x),
                  tronq=rep(F,dim_x),paramtronq=array(0,dim=c(2,dim_x))){
#---------------------------------------------------------------------------|
#Tirage aleatoire LHS de N jeux de parametres avec prise en compte de       |
#contraintes entre eux (croissance ou decroissance)                         |
#  References:                                                              |
#  Stein, M. 1987. Technometrics 29:143-151                                 |
#  Iman and Conover. 1980.                                                  |
#       Communications in Statistics: Theory and Methods A9: 1749-1874      |
#  McKay, Conover and Beckman. 1979. Technometrics 21: 239-245              |
#  Petelet et al., 2009. http://fr.arxiv.org/abs/0909.0329                  |
#---------------------------------------------------------------------------|
#Arguments obligatoires :                                                   |
#               dim_x : nombre de parametres d'entree dans le modele        |
#                                                                           |
#Arguments facultatifs :                                                    |
#             nom : vecteur des noms des parametres simules                 |
#               (defini comme : c("V1","V2","V3",...))                      |
#             N : nombre de jeux de simulations                             |
#             lois : vecteur contenant les types de distribution de proba   |
#               pour chaque entree                                          |
#               0=uniforme ; 1=normale ; 2=lognormale ; 3=weibull           | 
#               4=exponentielle ; 5=beta ; 6=triangulaire ; 7=trapezoidale  |
#               10=gumbel                                                   |
#               Par defaut, on prend la loi uniforme                        |
#             paramlois : tableau avec les parametres de chaque loi (max=4) |
#               pour chaque entree (range par colonne) :                    |
#               (min,max,0,0) pour uniforme (par dfaut : min=0, max=1)     |
#               (moy,ecart-type,0,0) pour normale,                          |
#               (moy du log, ecart-type du log,0,0) pour lognormale,        |
#               (forme,echelle,0,0) pour Weibull,                           |
#               (lambda,0,0,0) pour exponentielle,                          |
#               (shape1, shape2,0,0) pour beta,                             |
#               (min,mode,max,0) pour triangulaire,                         |
#               (min,mode1,mode2,max) pour trapezoidale,                    |
#               (mode,echelle,0,0) pour Gumbel                              |
#             tronq : vecteur pour selectionner ou non une loi tronquee     |
#               TRUE pour loi tronquee, FALSE sinon (par dfaut)            |
#             paramtronq : tableau avec les parametres de troncature de     |
#               chaque loi : (min,max) range par colonne                    |
#             contraint : vecteur contenant les types de contraintes a      |
#               imposer pour chaque Vi                                      |
#               0=tirage LHS ; 1=contrainte de croissance ; -1=contrainte   |
#               de decroissance.                                            |
#               Par defaut, on realise un tirage LHS.                       |
#---------------------------------------------------------------------------|
#Sortie : la matrice des N simulations des dim_x parametres                 |
#---------------------------------------------------------------------------|
# Auteurs : M. Petelet (DEN/SAC/DM2S/SEMT/LTA)                              |
#           B. Iooss (DEN/CAD/DER/SESI/LCFR)                                |
#---------------------------------------------------------------------------|

# !!! WARNING !!!! : le LHS, la troncature et les contraintes ne 
#                    s'appliquent pas a la loi trapezoidale

# *********************************************************************
# FONCTION UTILISEE :
#      truncated.R (fonctions pour lois tronquees)
# *********************************************************************
# LIBRAIRIES REQUISES (A INSTALLER)
  library(triangle)
  library(evd) # Gumbel
# *********************************************************************

  for(i in 1:dim_x){ # si le nom du x n'est pas defini
    if(is.na(nom[i])==TRUE) nom[i]<-paste("V",i,sep="")}

  x<-matrix(0,nr=N,nc=dim_x,dimnames=list(1:N,nom))

#on complete les bornes des lois uniformes, triangle et trapeze si necessaire
    for (i in 1:dim_x) {
	if (lois[i]==0 && paramlois[1,i]>=paramlois[2,i]) paramlois[2,i]<-1
	else if (lois[i]==6 && paramlois[1,i]>=paramlois[3,i]) {
		paramlois[2,i]<-0.5
		paramlois[3,i]<-1
	}
	else if (lois[i]==7 && paramlois[1,i]>=paramlois[4,i]) {
		paramlois[2,i]<-0.25
		paramlois[3,i]<-0.75
		paramlois[4,i]<-1
	}}

# la 1ere variable ne peut pas etre contrainte
  contraint[1] <- 0

  i <- 1
  while (i <= dim_x){ # repeter tant que i est < ou = au nb de parametres- 
    tirOK <- FALSE
#    print(paste("Echantillonage de la variable ",i))
    ran<-matrix(runif(N),nrow=N,ncol=1) # tirages aleatoires uniformes

  # debut methode de Stein..............................................

    idx<-sample(1:N)
    P<-(idx-ran)/N # probabilite de la fonction de repartition 

    if (lois[i]==0){ # loi uniforme
      if (tronq[i]==F) x[,i] <- qunif(P,min=paramlois[1,i],max=paramlois[2,i])
      else x[,i] <- q.trunc.distr(P,'unif',trunc.int=paramtronq[,i],min=paramlois[1,i],max=paramlois[2,i])}

    if (lois[i]==1){ # loi normale
      if (tronq[i]==F) x[,i] <-  qnorm(P,mean=paramlois[1,i],sd=paramlois[2,i])
      else x[,i] <- q.trunc.distr(P,'norm',trunc.int=paramtronq[,i],mean=paramlois[1,i],sd=paramlois[2,i])}

    if (lois[i]==2){ # loi lognormale
      if (tronq[i]==F) x[,i] <- qlnorm(P,meanlog=paramlois[1,i],sdlog=paramlois[2,i])
      else x[,i] <- q.trunc.distr(P,'lnorm',trunc.int=paramtronq[,i],meanlog=paramlois[1,i],sdlog=paramlois[2,i])}

    if (lois[i]==3){ # loi weibull
      if (tronq[i]==F) x[,i] <- qweibull(P,shape=paramlois[1,i],scale=paramlois[2,i])
      else x[,i] <- q.trunc.distr(P,'weibull',trunc.int=paramtronq[,i],shape=paramlois[1,i],scale=paramlois[2,i])}

    if (lois[i]==10){ # loi gumbel
      if (tronq[i]==F) x[,i] <- qgumbel(P,loc=paramlois[1,i],scale=paramlois[2,i])
      else x[,i] <- q.trunc.distr(P,'gumbel',trunc.int=paramtronq[,i],loc=paramlois[1,i],scale=paramlois[2,i])}

    if (lois[i]==4){ # loi exponentielle
      if (tronq[i]==F) x[,i]<- qexp(P,rate=paramlois[1,i])
      else x[,i] <- q.trunc.distr(P,'exp',trunc.int=paramtronq[,i],rate=paramlois[1,i])}

    if (lois[i]==5){ # loi beta
      if (tronq[i]==F) x[,i]<- qbeta(P,shape1=paramlois[1,i],shape2=paramlois[2,i])
      else x[,i] <- q.trunc.distr(P,'beta',trunc.int=paramtronq[,i],shape1=paramlois[1,i],shape2=paramlois[2,i])}

    if (lois[i]==6) { # loi triangulaire
      if (tronq[i]==F) x[,i]<- qtriangle(P,a=paramlois[1,i],b=paramlois[3,i],c=paramlois[2,i])
      else x[,i] <- q.trunc.distr(P,'triangle',trunc.int=paramtronq[,i],a=paramlois[1,i],b=paramlois[3,i],c=paramlois[2,i])}

    if (lois[i]==7) { # loi trapezoidale ( !!! pas LHS !!! )
      norm<-1/(paramlois[2,i]-paramlois[1,i])
      normp<-1/(paramlois[4,i]-paramlois[3,i])
      j<-1
      while (j<=N) {
        aux<-runif(1,min=paramlois[1,i],max=paramlois[4,i])
        auxp<-runif(1)
        if (aux<=paramlois[2,i]) {
          if (auxp<=norm*(aux-paramlois[1,i])) {
            x[j,i]<-aux
            j<-j+1}}
        else if (aux>=paramlois[3,i]) {
          if (auxp<=normp*(paramlois[4,i]-aux)) {
            x[j,i]<-aux
            j<-j+1}}
        else {
          x[j,i]<-aux
          j<-j+1}}}

  # fin methode de Stein................................................

  # calcul de la matrice de compatibilit
    compat<-(x[,i] %*% t(1/x[,i-1]))-1
  # application de la fonction de Heaviside (en 2 etapes):  
    compat[which(compat>=0)]<-1
    compat[which(compat<0)]<-0
    if (contraint[i]==-1) compat<-(1-compat)
    if (contraint[i]==0) tirOK <- TRUE #0 : tirage LHS sans contrainte
    else{ #1 ou -1 : tirage LHS avec contrainte
      if (sum(diag(compat))==N) tirOK <- TRUE # La contrainte est respecte 
      else{ # Il faut faire quelques permutations
        if (contraint[i]==1) { #1 : contrainte de croissance
          ordre<-order(x[,i-1],decreasing=T)
        }
      
        if (contraint[i]==-1) { #-1 : contrainte de decroissance
          ordre<-order(x[,i-1],decreasing=F)
        }
  
      # Conditions ncessaire et suffisante pour que le tirage LHS puisse 
      # fournir au moins un rarrangement satisfaisant la contrainte :
      # la diag de X.Xt doit comprendre au moins 1...N 
        if (min(sort(diag(compat %*% t(compat)))-1:N) >= 0){
          for(j in 1:N){ 
          # ordre[j] est la ligne la plus urgente a placer
            possibl<-which(compat[,ordre[j]]==1) 
            if (length(possibl)==1) retenu<-possibl
            else retenu<-sample(possibl)[1]
            compat[retenu,] <- 0
            compat[,ordre[j]] <- 0
            compat[retenu,ordre[j]] <- 1
          }
        
        # which renvoie les indices de la matrice = 1
          x[,i]<-x[(which(compat==1,arr.ind=T))[,1],i]
        
        # calcul de la matrice de compatibilit
          compat<-(x[,i] %*% t(1/x[,i-1]))-1
        
        # application de la fonction de Heaviside  
          compat[which(compat>=0)]<-1
          compat[which(compat<0)]<-0 
          if (contraint[i]==-1) compat<-(1-compat)
        }
   
      # fin tirage LHS sous contrainte de croissance 
        if ((contraint[i]==1)&(sum(diag(compat))==N)) tirOK<-TRUE 
      
      # fin tirage LHS sous contrainte de decroissance 
        if ((contraint[i]==-1)&(sum(diag(compat))==N)) tirOK<-TRUE
      }
    }
    if (tirOK==TRUE) i <- i+1
  } # fin de repeter tant que i est < ou = au nb de parametres------------

  return(x)

}
