samplingOptLHS<-function(dim_x,nom=c(NA),N=1,lois=rep(0,dim_x),
                   paramlois=array(0,dim=c(4,dim_x)),
                   tronq=rep(F,dim_x),paramtronq=array(0,dim=c(2,dim_x)),
                   optimal="maximin",dup=1,pop=100,gen=4,pMut=0.1){

#---------------------------------------------------------------------------|
#Tirage aleatoire d'un plan LHS optimal de N jeux de parametres             |
#  Le plan peut etre maximin, distance-optimal ou S-optimal                 |
#---------------------------------------------------------------------------|
#Arguments obligatoires :                                                   |
#               dim_x : nombre de parametres d'entree dans le modele        |
#                                                                           |
#Arguments facultatifs :                                                    |
#             nom : vecteur des noms des parametres simules                 |
#               (defini comme : c("V1","V2","V3",...))                      |
#             N : nombre de jeux de simulations                             |
#             lois : vecteur contenant les types de distribution de proba   |
#               pour chaque entree                                          |
#               0=uniforme ; 1=normale ; 2=lognormale ; 3=weibull           | 
#               4=exponentielle ; 5=beta ; 6=triangulaire ; 7=trapezoidale  |
#               10=gumbel                                                   |
#               Par defaut, on prend la loi uniforme                        |
#             paramlois : tableau avec les parametres de chaque loi (max=4) |
#               pour chaque entree (range par colonne) :                    |
#               (min,max,0,0) pour uniforme (par dfaut : min=0, max=1)     |
#               (moy,ecart-type,0,0) pour normale,                          |
#               (moy du log, ecart-type du log,0,0) pour lognormale,        |
#               (forme,echelle,0,0) pour Weibull,                           |
#               (lambda,0,0,0) pour exponentielle,                          |
#               (shape1, shape2,0,0) pour beta,                             |
#               (min,mode,max,0) pour triangulaire,                         |
#               (min,mode1,mode2,max) pour trapezoidale,                    |
#               (mode,echelle,0,0) pour Gumbel                              |
#             tronq : vecteur pour selectionner ou non une loi tronquee     |
#               TRUE pour loi tronquee, FALSE sinon (par dfaut)            |
#             paramtronq : tableau avec les parametres de troncature de     |
#               chaque loi : (min,max) range par colonne                    |
#             optimal : type d'optimalite pour le LHS (par defaut "maximin")|
#               "distance" pour distance-optimal, "S" pour S-optimal        |
#             dup : facteur pour le nb de points candidats dans les fcts    |
#               maximinLHS (plan maximin) et improvedLHS (plan dist-optimal)|
#             pop, gen, pMut : options de la fct geneticLHS (plan S-optimal)|
#               Taper help(geneticLHS) pour en savoir plus                  |
#---------------------------------------------------------------------------|
#Sortie : la matrice des N simulations des dim_x parametres                 |
#---------------------------------------------------------------------------|
#      Auteur : B. Iooss                                                    |
#---------------------------------------------------------------------------|

# !!! WARNING !!!! : le LHS et la la troncature ne s'appliquent pas a la 
#                    loi trapezoidale

# *********************************************************************
# FONCTION UTILISEE :
#      truncated.R (fonctions pour lois tronquees)
# *********************************************************************
# LIBRAIRIES REQUISES (A INSTALLER)
  library(triangle)
  library(lhs)
  library(evd) # Gumbel
# *********************************************************************

# si le nom du x n'est pas defini
  for(i in 1:dim_x){
    if(is.na(nom[i])==TRUE) nom[i]<-paste("V",i,sep="")}
  
#on complete les bornes des lois uniformes, triangle et trapeze si necessaire
  for (i in 1:dim_x) {
	if (lois[i]==0 && paramlois[1,i]>=paramlois[2,i]) paramlois[2,i]<-1
	else if (lois[i]==6 && paramlois[1,i]>=paramlois[3,i]) {
		paramlois[2,i]<-0.5
		paramlois[3,i]<-1
	}
	else if (lois[i]==7 && paramlois[1,i]>=paramlois[4,i]) {
		paramlois[2,i]<-0.25
		paramlois[3,i]<-0.75
		paramlois[4,i]<-1
	}}
  
# Plan LHS maximin
  if (optimal=="distance") x <- improvedLHS(n=N,k=dim_x,dup=dup)
# Plan LHS S-optimal
  if (optimal=="S") x <- geneticLHS(n=N,k=dim_x,pop=pop,gen=gen,pMut=pMut)
# Plan LHS maximin
  else x <- maximinLHS(n=N,k=dim_x,dup=dup)

  x <- matrix(x,nr=N,nc=dim_x,dimnames=list(1:N,nom))

# Tirages aleatoires uniformes
  ran<-matrix(runif(N*dim_x),nrow=N,ncol=dim_x)

  for (i in 1:dim_x){

    idx<-sample(1:N)
    P<-(idx-ran[,i])/N # probabilite de la fonction de repartition 

    if (lois[i]==0){ # loi uniforme
      if (tronq[i]==F) x[,i] <- qunif(P,min=paramlois[1,i],max=paramlois[2,i])
      else x[,i] <- q.trunc.distr(P,'unif',trunc.int=paramtronq[,i],min=paramlois[1,i],max=paramlois[2,i])}

    if (lois[i]==1){ # loi normale
      if (tronq[i]==F) x[,i] <-  qnorm(P,mean=paramlois[1,i],sd=paramlois[2,i])
      else x[,i] <- q.trunc.distr(P,'norm',trunc.int=paramtronq[,i],mean=paramlois[1,i],sd=paramlois[2,i])}

    if (lois[i]==2){ # loi lognormale
      if (tronq[i]==F) x[,i] <- qlnorm(P,meanlog=paramlois[1,i],sdlog=paramlois[2,i])
      else x[,i] <- q.trunc.distr(P,'lnorm',trunc.int=paramtronq[,i],meanlog=paramlois[1,i],sdlog=paramlois[2,i])}

    if (lois[i]==3){ # loi weibull
      if (tronq[i]==F) x[,i] <- qweibull(P,shape=paramlois[1,i],scale=paramlois[2,i])
      else x[,i] <- q.trunc.distr(P,'weibull',trunc.int=paramtronq[,i],shape=paramlois[1,i],scale=paramlois[2,i])}

    if (lois[i]==10){ # loi gumbel
      if (tronq[i]==F) x[,i] <- qgumbel(P,loc=paramlois[1,i],scale=paramlois[2,i])
      else x[,i] <- q.trunc.distr(P,'gumbel',trunc.int=paramtronq[,i],loc=paramlois[1,i],scale=paramlois[2,i])}

    if (lois[i]==4){ # loi exponentielle
      if (tronq[i]==F) x[,i]<- qexp(P,rate=paramlois[1,i])
      else x[,i] <- q.trunc.distr(P,'exp',trunc.int=paramtronq[,i],rate=paramlois[1,i])}

    if (lois[i]==5){ # loi beta
      if (tronq[i]==F) x[,i]<- qbeta(P,shape1=paramlois[1,i],shape2=paramlois[2,i])
      else x[,i] <- q.trunc.distr(P,'beta',trunc.int=paramtronq[,i],shape1=paramlois[1,i],shape2=paramlois[2,i])}

    if (lois[i]==6) { # loi triangulaire
      if (tronq[i]==F) x[,i]<- qtriangle(P,a=paramlois[1,i],b=paramlois[3,i],c=paramlois[2,i])
      else x[,i] <- q.trunc.distr(P,'triangle',trunc.int=paramtronq[,i],a=paramlois[1,i],b=paramlois[3,i],c=paramlois[2,i])}

    if (lois[i]==7) { # loi trapezoidale ( !!! pas LHS !!! )
      norm<-1/(paramlois[2,i]-paramlois[1,i])
      normp<-1/(paramlois[4,i]-paramlois[3,i])
      j<-1
      while (j<=N) {
        aux<-runif(1,min=paramlois[1,i],max=paramlois[4,i])
        auxp<-runif(1)
        if (aux<=paramlois[2,i]) {
          if (auxp<=norm*(aux-paramlois[1,i])) {
            x[j,i]<-aux
            j<-j+1}}
        else if (aux>=paramlois[3,i]) {
          if (auxp<=normp*(paramlois[4,i]-aux)) {
            x[j,i]<-aux
            j<-j+1}}
        else {
          x[j,i]<-aux
          j<-j+1}}}
  }
  
  return(x)

}
