sensibilite<-function(x,y,nseg=1,nsegy=0,xnom="V",ynom="y",
                      nom=c(NA),affiche=TRUE){

#---------------------------------------------------------------
#Fonction qui calcule des indices de sensibilite bases sur des  |
#echantillons ("sampling-based sensitivity indices")            |
#                                                               |
# 1) On calcule les coefficients de correlation et de           |
#regression entre des parametres x et des variables y :         |
#coef de correlation (Pearson), de regression standard (SRC) et |
#de correlation partielle (PCC).                                |
#Le coefficient de determination R2 (entre les x et y) est donne|
#pour (in)valider l'hypothese de linearite.                     |
#                                                               |
# 2) On calcule les SRC et PCC des variables centrees-reduites  |
#(a utiliser si les ordres de grandeur des parametres x sont    |
#tres differents)                                               |
#                                                               |
# 3) La fonction calcule les coefficients bases sur les rangs:  |
#coef de correlation des rangs (Spearman), de regression        |
#standard des rangs (SRRC), de correlation partielle standard   |
#des rangs (PRCC).                                              |
#Le coefficient de determination R2* (entre les rangs des x,y)  |
#est donne pour (in)valider l'hypothese de monotonie.           |
#                                                               |
# 4) On calcule les indices bases sur des tests statistiques    |
#et le decoupage du domaine des entrees :                       |
#test CL (Common Locations), test CMN (Common Means),           |
#test CMD (Common Medians), test CV (Common Variances) et       |
# test SI (Statistical Independance)                            |
#                                                               |
#Tous ces indices sont tries par ordre decroissant              |
#                                                               |
#Cette fonction affiche aussi les graphiques scatterplots entre |
#les y et chaque parametre x                                    | 
#---------------------------------------------------------------|
#Arguments                                                      |
#obligatoires : x : matrice des variables d'entree              |
#               y : matrice des variables de sortie             |
#                                                               |
#Arguments                                                      |
#facultatifs : nseg : nombre de partitions sur les entrees      |
#                pour les indices bases sur les tests stats     |
#                Par defaut : nseg=1 (pas de tests)             |
#              nsegy : nb de partition sur la sortie (test SI)  |
#                Par defaut : nsegy=nseg                        |
#              xnom : "nom" pour les variables d'entree         |
#                (on utilise xnom suivi d'un numero par entree) |
#              ynom : "nom" pour les variables de sortie        |
#                (on utilise ynom suivi d'un numero par sortie) |
#                ynom est utilise si la variable n'a pas de nom |
#              nom : vecteur des noms des x et y (6 caracteres) |
#                (xnom et ynom ne sont alors pas pris en compte)|
#              affiche : type d'affichage                       |
#                TRUE -> scatterplot + resultats detailles      |
#                FALSE->resultats resumes (OK si bcp de sorties)|
#---------------------------------------------------------------|
#Sortie : Affiche tous les indices de sensibilite entre les y et|
#           chaque x : indices lineaires (Pearson, SRC, PCC),   |
#           indices SRC et PCC des x centrees-reduites,         |
#           indices monotones (Spearman, SRRC, PRCC),           |
#           indices CL, CMN, CMD, CV, SI                        |
#         Affiche les R2, R2 ajuste, R2*, R2* ajuste pour       |
#           (in)valider les hypotheses lineaire et monotone     |
#         Ces resultats sont dans le fichier res-SENSIBILITE.out|
#         Si affiche=TRUE, trace les graphiques scatterplots    |
#           entre les y et chaque parametre x                   |
#         Si affiche==FALSE, on conserve les vecteurs etmatrices|
#           R2_n R2aju_n pear_n src_n pcc_n R2star_n R2staraju_n|
#           spear_n srrc_n prcc_n CL_n CMN_n CMD_n CV_n SI_n    |
#           qui contiennent les coefficients de sensibilite     |
#----------------------------------------------------------------
# Auteurs : B. Iooss, A. Ducellier, A. De Crecy, D. Barthel     |
#----------------------------------------------------------------

# *********************************************************************
# LIBRAIRIES REQUISES (A INSTALLER)
  library(car)
# *********************************************************************

#INITIALISATIONS

x<-as.matrix(x)
nr<-dim(x)[1]
nx<-dim(x)[2]

y<-as.matrix(y)
ny<-dim(y)[2]

# si la variable "nom" du x n'est pas definie
for(i in 1:nx){
#  if(is.na(nom[i])==TRUE) nom[i]<-dimnames(x)[[2]][i]
  if(is.na(nom[i])==TRUE){
    if (is.null(dimnames(x)[[2]][i])==TRUE){
       nom[i]<-paste(xnom,as.character(i),sep="")}
    else nom[i]<-dimnames(x)[[2]][i]}}

x<-matrix(c(x),nr=nr,nc=nx,dimnames=list(1:nr,nom[1:nx]))

# si la variable "nom" du y n'est pas definie
for(i in 1:ny){
  if(is.na(nom[nx+i])==TRUE){
    if (is.null(dimnames(y)[[2]][i])==TRUE){
       nom[nx+i]<-paste(ynom,as.character(i),sep="")}
    else nom[nx+i]<-dimnames(y)[[2]][i]}}

y<-matrix(c(y),nr=nr,nc=ny,dimnames=list(1:nr,nom[(nx+1):(nx+ny)]))

if (affiche==FALSE){
  R2_n <- rep(0,ny)
  R2aju_n <- rep(0,ny)
  pear_n <- matrix(0,nr=nx,nc=ny,dimnames=list(nom[1:nx],nom[(nx+1):(nx+ny)]))
  r_n <- matrix(0,nr=nx,nc=ny,dimnames=list(nom[1:nx],nom[(nx+1):(nx+ny)]))
  p_n <- matrix(0,nr=nx,nc=ny,dimnames=list(nom[1:nx],nom[(nx+1):(nx+ny)]))
  R2star_n <- rep(0,ny)
  R2staraju_n <- rep(0,ny)
  spear_n <- matrix(0,nr=nx,nc=ny,dimnames=list(nom[1:nx],nom[(nx+1):(nx+ny)]))
  u_n <- matrix(0,nr=nx,nc=ny,dimnames=list(nom[1:nx],nom[(nx+1):(nx+ny)]))
  q_n <- matrix(0,nr=nx,nc=ny,dimnames=list(nom[1:nx],nom[(nx+1):(nx+ny)]))
  if (nseg>1){
    CL_n <- matrix(0,nr=nx,nc=ny,dimnames=list(nom[1:nx],nom[(nx+1):(nx+ny)]))
    CMN_n <- matrix(0,nr=nx,nc=ny,dimnames=list(nom[1:nx],nom[(nx+1):(nx+ny)]))
    CMD_n <- matrix(0,nr=nx,nc=ny,dimnames=list(nom[1:nx],nom[(nx+1):(nx+ny)]))
    CV_n <- matrix(0,nr=nx,nc=ny,dimnames=list(nom[1:nx],nom[(nx+1):(nx+ny)]))
    SI_n <- matrix(0,nr=nx,nc=ny,dimnames=list(nom[1:nx],nom[(nx+1):(nx+ny)]))
  }
}
#Creation de la formule de regression
#f="y~" # Il faut une ordonn�e � l'origine
#f=paste(f,xnom,as.character(1),sep="")
#if (nx>1){
#  for(i in 2:nx) f=paste(f,"+",xnom,as.character(i),sep="")}
#formule=as.formula(f)

#Creation du fichier de resultats
FOUT <- file("res-SENSIBILITE.out","w")

#*****************************************************************
#************ BOUCLE sur les VARIABLES de SORTIE *****************
#*****************************************************************

for (i_y in 1:ny){

#  yy<-as.vector(y[,i_y])
  yy<-matrix(y[,i_y],nr=nr,nc=1,,dimnames=list(1:nr,nom[nx+i_y]))

#Creation de la formule de regression

  f=paste(nom[nx+i_y],"~") # Il faut une ordonn�e � l'origine
  f=paste(f,nom[1],sep="")
  if (nx>1){
    for(i in 2:nx) f=paste(f,"+",nom[i],sep="")}
  formule=as.formula(f)

#*****************************************************************
# SCATTERPLOTS
  if (affiche == TRUE){
    ngraph <- ceiling(nx/20) # 20 plots au max par fenetre graphique
    ngraphy <- min(floor(sqrt(nx)+0.999),5)
    ngraphx <- min(floor(sqrt(nx)+0.5),4)

    for (ig in 1:ngraph){
#    postscript(file="scatterplot.ps")
      x11()
      op <- par(mfcol=c(ngraphx,ngraphy))
      ix<-1
      iy<-1
      i<-1
      while ((i <= ngraphx*ngraphy) & (i+(ig-1)*20 <= nx)){
        par(mfg=c(ix,iy))
        plot(x[,i+(ig-1)*20],yy,xlab=nom[i+(ig-1)*20],ylab=nom[nx+i_y])
        lines(lowess(x[,i+(ig-1)*20],yy), col = 2, lty = 2, lwd = 2)
        if (iy<ngraphy) iy<-iy+1
        else{
          iy<-1
          ix<-ix+1}
        i<-i+1}
      par(op)
      mtext("Scatterplots",line=3)
#    dev.off()
    }
  }
  
#***************************************************************************
#----------------------------------------
#Calcul des coefficients sur les valeurs |
#----------------------------------------

#Calcul des correlations de Pearson  
#--------------------------------------------

  pear<-cor(x,yy)

# classement des parametres par ordre decroissant de sensibilite
# n_pear[i] est le numero du parametre qui se trouve au ieme rang

  n_pear<-rep(0,nx)

  rpear<-rank(abs(pear),na.last=F)
#  revpear<-rev(sort(abs(pear),na.last=F))

#  for (i in 1:nx){
#      for  (j in 1:nx) if (abs(revpear[i]-abs(pear[j]))<=0.000001) n_pear[i] = j }
  for (i in 1:nx) n_pear[nx+1-rpear[i]] <- i

  pear<-format(pear,digits=4)

#Calcul des coefficients de regression standard 
#-----------------------------------------------

#initialisations
  r<-rep(0,nx)

  modele<-lm(formule,data=data.frame(x,y=yy))
  a<-modele$coefficients

  for (i in 1:nx) r[i] <- a[i+1]*sd(x[,i])/sd(yy)

# classement des parametres par ordre decroissant de sensibilite
# n_r[i] est le numero du parametre qui se trouve au ieme rang

  n_r<-rep(0,nx)

  rr<-rank(abs(r),na.last=F)
  revr<-rev(sort(abs(r),na.last=F))

#  for (i in 1:nx){
#      for  (j in 1:nx)
#           if (abs(revr[i]-abs(r[j]))<=0.000001) n_r[i] = j }
  for (i in 1:nx) n_r[nx+1-rr[i]] <- i
  
  r<-format(r,digits=4)

#Calcul du coefficient de determination (R2)
#--------------------------------------------

  R2 <- summary(modele)$r.squared
  R2 <- format(R2,digits=4)
  R2aju <- summary(modele)$adj.r.squared
  R2aju <- format(R2aju,digits=4)

#Calcul des correlations partielles standard 
#---------------------------------------------

#Calcul de e (residus)
  e <- modele$residuals

#Calcul des ei

  E<-matrix(0,nr=nr,nc=nx)

  for(i in 1:nx){

#Calcul de la nouvelle matrice

    x2<-x
    x2[,i]<-yy
    yy1<-matrix(x[,i],nr=nr,nc=1,dimnames=list(1:nr,nom[nx+i_y]))
    modele<-lm(formule,data=data.frame(x2,y=yy1))

#    if(i>1){for(j in 1:(i-1)){x2[,j]<-x[,j]}}
#    if(i<nx){for(j in i:(nx-1)){x2[,j]<-x[,(j+1)]}}
#    x2[,nx]<-yy
#    x2<-matrix(x2,nr=nr,nc=nx,dimnames=list(1:nr,dimnames(x)[[2]]))
#    modele<-lm(formule,data=data.frame(x2,y=x[,i]))

    a<-modele$coefficients

  
    E[,i] <- modele$residuals
  }

  p<-rep(0,nx)

  for(i in 1:nx){
    p[i]<- -(crossprod(e,E[,i]))/((sqrt(crossprod(e,e)))*(sqrt(crossprod(E[,i],E[,i]))))
  }

# classement des parametres par ordre decroissant de sensibilite
# n_p[i] est le numero du parametre qui se trouve au ieme rang

  n_p<-rep(0,nx)

  rp<-rank(abs(p),na.last=F)
#  revp<-rev(sort(abs(p),na.last=F))

#  for (i in 1:nx){
#     for  (j in 1:nx) if (abs(revp[i]-abs(p[j]))<=0.000001) n_p[i] = j }

  for (i in 1:nx) n_p[nx+1-rp[i]] <- i
  
  p<-format(p,digits=4)

#--------------------------------------------
#Calcul des coefficients sur les rangs       |
#--------------------------------------------

#Calcul de la matrice des rangs

  xr<-matrix(0,nr=nr,nc=nx)
  for(i in 1:nx){xr[,i]<-rank(x[,i])}
  xr<-matrix(xr,nr=nr,nc=nx,dimnames=list(1:nr,nom[1:nx]))

  yr<-matrix(rank(yy),nr=nr,nc=1,dimnames=list(1:nr,nom[nx+i_y]))

#Calcul des correlations de Spearman     
#---------------------------------------

#spear<-cor(xr,yr)
  spear<-cor(x,yy,method="spearman")

# classement des parametres par ordre decroissant de sensibilite
# n_spear[i] est le numero du parametre qui se trouve au ieme rang

  n_spear<-rep(0,nx)

  rspear<-rank(abs(spear),na.last=F)
#  revspear<-rev(sort(abs(spear),na.last=F))

#  for (i in 1:nx){
#    for  (j in 1:nx)
#         if (abs(revspear[i]-abs(spear[j]))<=0.000001) n_spear[i] = j }

  for (i in 1:nx) n_spear[nx+1-rspear[i]] <- i
  
spear<-format(spear,digits=4)

#Calcul des coefficients de regression standard des rangs 
#--------------------------------------------------------

#initialisations
  u<-rep(0,nx)

  modele<-lm(formule,data=data.frame(xr,y=yr))
  a<-modele$coefficients


  for (i in 1:nx) u[i] <- a[i+1]*sd(xr[,i])/sd(yr)

# classement des parametres par ordre decroissant de sensibilite
# n_u[i] est le numero du parametre qui se trouve au ieme rang

  n_u <- rep(0,nx)

  ru<-rank(abs(u),na.last=F)
#  revu<-rev(sort(abs(u),na.last=F))

#  for (i in 1:nx){
#      for  (j in 1:nx) if (abs(revu[i]-abs(u[j]))<=0.000001) n_u[i] = j }

  for (i in 1:nx) n_u[nx+1-ru[i]] <- i
  
  u<-format(u,digits=4)

#Calcul du coefficient de determination des rangs (R2*)
#------------------------------------------------------

  R2star <- summary(modele)$r.squared
  R2star <- format(R2star,digits=4)
  R2staraju <- summary(modele)$adj.r.squared
  R2staraju <- format(R2staraju,digits=4)

#Calcul des correlations partielles des rangs
#------------------------------------------------

#Calcul de e (residus)
  e <- modele$residuals

#Calcul des ei

  E<-matrix(0,nr=nr,nc=nx)

  for(i in 1:nx){

#Calcul de la nouvelle matrice

    xr2<-xr
    xr2[,i]<-yr
    yy1<-matrix(xr[,i],nr=nr,nc=1,dimnames=list(1:nr,nom[nx+i_y]))
    modele<-lm(formule,data=data.frame(xr2,y=yy1))

#    if(i>1){for(j in 1:(i-1)){xr2[,j]<-xr[,j]}}
#    if(i<nx){for(j in i:(nx-1)){xr2[,j]<-xr[,(j+1)]}}
#    xr2[,nx]<-yr
#    xr2<-matrix(xr2,nr=nr,nc=nx,dimnames=list(1:nr,dimnames(x)[[2]]))
#    modele<-lm(formule,data=data.frame(xr2,y=xr[,i]))

    a<-modele$coefficients

    E[,i] <- modele$residuals
  }

  q<-rep(0,nx)

  for(i in 1:nx){
    q[i]<- -(crossprod(e,E[,i]))/((sqrt(crossprod(e,e)))*(sqrt(crossprod(E[,i],E[,i]))))
  }

# classement des parametres par ordre decroissant de sensibilite
# n_q[i] est le numero du parametre qui se trouve au ieme rang

  n_q<-rep(0,nx)

  rq<-rank(abs(q),na.last=F)
#  revq<-rev(sort(abs(q),na.last=F))

#  for (i in 1:nx){
#    for  (j in 1:nx) if (abs(revq[i]-abs(q[j]))<=0.000001) n_q[i] = j}

  for (i in 1:nx) n_q[nx+1-rq[i]] <- i
  
  q<-format(q,digits=4)

#***************************************************************************
#-----------------------------------------
#Calcul sur les valeurs centrees-reduites |
#-----------------------------------------

#Calcul de la matrice centree-reduite

  xm<-matrix(0,nr=nr,nc=nx)
  for(i in 1:nx) xm[,i]<-(x[,i]-mean(x[,i]))/sd(x[,i])
  xm<-matrix(xm,nr=nr,nc=nx,dimnames=list(1:nr,nom[1:nx]))
 
  ym<-matrix((yy-mean(yy))/sd(yy),nr=nr,nc=1,dimnames=list(1:nr,nom[nx+i_y]))

#Calcul des coefficients de regression standard 
#-----------------------------------------------

#initialisations
  rm<-rep(0,nx)

  modele<-lm(formule,data=data.frame(xm,y=ym))
  a<-modele$coefficients

  for (i in 1:nx) rm[i] <- a[i+1]*sd(xm[,i])/sd(ym)

# classement des parametres par ordre decroissant de sensibilite
# n_rm[i] est le numero du parametre qui se trouve au ieme rang

  n_rm<-rep(0,nx)

  rrm<-rank(abs(rm),na.last=F)
#  revrm<-rev(sort(abs(rm),na.last=F))

#  for (i in 1:nx){
#    for  (j in 1:nx)
#         if (abs(revrm[i]-abs(rm[j]))<=0.000001) n_rm[i] = j  }

  for (i in 1:nx) n_rm[nx+1-rrm[i]] <- i
  
  rm<-format(rm,digits=4)

#Calcul du coefficient de determination (R2)
#--------------------------------------------

  R2m <- summary(modele)$r.squared
  R2m <- format(R2m,digits=4)
  R2ajum <- summary(modele)$adj.r.squared
  R2ajum <- format(R2ajum,digits=4)

#Calcul des correlations partielles standard 
#---------------------------------------------

#Calcul de e (residus)
  e <- modele$residuals

#Calcul des ei

  E<-matrix(0,nr=nr,nc=nx)

  for(i in 1:nx){

#Calcul de la nouvelle matrice

    x2m<-xm
    x2m[,i]<-ym
    yy1<-matrix(x2m[,i],nr=nr,nc=1,dimnames=list(1:nr,nom[nx+i_y]))
    modele<-lm(formule,data=data.frame(x2m,y=yy1))

#    if(i>1){for(j in 1:(i-1)){x2m[,j]<-xm[,j]}}
#    if(i<nx){for(j in i:(nx-1)){x2m[,j]<-xm[,(j+1)]}}
#    x2m[,nx]<-ym
#    x2m<-matrix(x2m,nr=nr,nc=nx,dimnames=list(1:nr,dimnames(x)[[2]]))
#    modele<-lm(formule,data=data.frame(x2m,y=xm[,i]))

    a<-modele$coefficients

    E[,i] <- modele$residuals
  }

  pm<-rep(0,nx)

  for(i in 1:nx){
    pm[i]<- -(crossprod(e,E[,i]))/((sqrt(crossprod(e,e)))*(sqrt(crossprod(E[,i],E[,i]))))
  }

# classement des parametres par ordre decroissant de sensibilite
# n_pm[i] est le numero du parametre qui se trouve au ieme rang

  n_pm<-rep(0,nx)

  rpm<-rank(abs(pm),na.last=F)
#  revpm<-rev(sort(abs(pm),na.last=F))

#  for (i in 1:nx){
#   for  (j in 1:nx) if (abs(revpm[i]-abs(pm[j]))<=0.000001) n_pm[i] = j}

  for (i in 1:nx) n_pm[nx+1-rpm[i]] <- i
  
  pm<-format(pm,digits=4)


#************************************************************
#Tests statistiques bases sur la segmentation des entrees
#************************************************************

if (nseg>1){

#*********************************************
# Creation des classes sur les entrees

  probs <- 0

  for (k in 1:nseg){
	probs <- append(probs,k/nseg)
  }

  g_mult <- matrix(0,nr,nx)

  for (i in 1:nx){
	quant <- quantile(x[,i],probs=probs,na.rm=T)
	for (j in 1:nr){
		for (k in 1:nseg){
			if (x[j,i]>=quant[k]){g_mult[j,i]=k}
		}
	}
  }
#la matrice g mult donne en colonne n°i ligne n°j (donc pour j-eme observation de la i-eme variable)
#le numero de la classe à laquelle l'observation a été affectée 

#**********************************************
# Common Locations (CL) - Kruskal-Wallis test
#Ici on pose Ho: les distributions conditionnelles sont egales

  CL.pvalue <- rep(0,nx)

  for (i in 1:nx){
	g=g_mult[,i]
	CL.pvalue[i] <- kruskal.test(yy,g)$p.value
  }

# classement des parametres par ordre decroissant de sensibilite
# n_krus[i] est le numero du parametre qui se trouve au ieme rang

  n_krus<-rep(0,nx)
  rkrus<-rank(-abs(CL.pvalue),na.last=F, ties.method = "random")
  for (i in 1:nx) n_krus[nx+1-rkrus[i]] <- i
  CL.pvalue<-format(CL.pvalue,digits=4)

#**********************************************
# Common Means (CMN) test

#Ici on pose Ho: les moyennes conditionnelles sont egales
# calcul de la statistique Fa (ou Fa_bc si une transformation de box cox est necessaire)
# du ANOVA F-test:

  CMN.pvalue=rep(0,nx)

  for (i in 1:nx){
	
	g=g_mult[,i]
	Somme_a=0
	
	for (iq in 1:nseg){
		#dans cette boucle iq represente la classe
		Somme_a=Somme_a+(((mean(yy[g==iq]))^2)*length(g[g==iq]))
	}
	
	Fa=((Somme_a-(nr)*((mean(yy))^2))/(nseg-1))/((as.numeric(yy)%*%as.numeric(yy)-Somme_a)/(nr-nseg))
	
	#On sait que dans ce cas, Fa suit une loi de Fisher(nseg-1,nr-nseg)
	CMN.pvalue[i]=1-pf(as.numeric(Fa),nseg-1,nr-nseg)
  }

# classement des parametres par ordre decroissant de sensibilite
# est le numero du parametre qui se trouve au ieme rang

  n_CMN<-rep(0,nx)

  rCMN<-rank(-abs(CMN.pvalue),na.last=F, ties.method = "random")

  for (i in 1:nx){
	n_CMN[nx+1-rCMN[i]]=i
  }

  CMN.pvalue<-format(CMN.pvalue,digits=4)

#****************************************
# Common Medians (CMD) test

#Ici on pose Ho: les medianes conditionnelles sont egales

  CMD.pvalue=rep(0,nx)

  for (i in 1:nx){
	g=g_mult[,i]
	
	N1=rep(0,nseg)
	N2=rep(0,nseg)
	
	for (iq in 1:nseg){
		Class_q_Data=yy[g==iq]
		Y_median=median(yy)
		N1[iq]=length(Class_q_Data[Class_q_Data > Y_median])
		N2[iq]=length(Class_q_Data[Class_q_Data <= Y_median])
	}
	
	Tstat=0
	
	vecto=c()
	
	for (iq in 1:nseg){
		NE_1q=(1/(nr))*(as.numeric(N1[iq])+as.numeric(N2[iq]))*(sum(N1))
		NE_2q=(1/(nr))*(as.numeric(N1[iq])+as.numeric(N2[iq]))*(sum(N2))
		class_term=(((as.numeric(N1[iq]) - as.numeric(NE_1q))^2)/(NE_1q))+(((as.numeric(N2[iq]) - as.numeric(NE_2q))^2)/(NE_2q))
		Tstat = Tstat + class_term
	}
	
	#Sous Ho, Tstat suit un Chisq à nseg-1 d.d.l (df)
	
	CMD.pvalue[i]=1-pchisq(Tstat,nseg - 1)
  }

# classement des parametres par ordre decroissant de sensibilite
# est le numero du parametre qui se trouve au ieme rang

  n_CMD<-rep(0,nx)

  rCMD<-rank(-abs(CMD.pvalue),na.last=F, ties.method = "random")
  for (i in 1:nx){
	n_CMD[nx+1-rCMD[i]]=i
  }

  CMD.pvalue<-format(CMD.pvalue,digits=4)

#****************************************
# Common Variances (Levene's test) test requires package "car"

#Ici on pose Ho: les variances conditionnelles sont egales

  CV.pvalue=rep(0,nx)

  for (i in 1:nx){
	g=g_mult[,i]
	Test_i=levene.test(c(yy),as.factor(g))
	CV.pvalue[i]=Test_i$"Pr(>F)"[1]
  }

# classement des parametres par ordre decroissant de sensibilite
# est le numero du parametre qui se trouve au ieme rang
  n_CV<-rep(0,nx)
  rCV<-rank(-abs(CV.pvalue),na.last=F, ties.method = "random")

  for (i in 1:nx){
	n_CV[nx+1-rCV[i]]=i
  }

  CV.pvalue<-format(CV.pvalue,digits=4)

#****************************************
# Statistical Independance (SI) test

#Ici on pose Ho: Y et x_i sont independantes
#On decoupe egalement le domaine de y en nsegy classes
  if (nsegy==0) nsegy=nseg

  SI.pvalue=rep(0,nx)

# Creation des classes sur la sortie

  if(nsegy > 1){

    probas <- 0

    for (k in 1:nsegy){
	probas <- append(probas,k/nsegy)
    }

    gy<- matrix(0,nr,1)

#    yyy=sort(yy)
    yyy=yy

    quant2 <- quantile(yyy,probs=probas,na.rm=T)
    for (j in 1:nr){
	for (k in 1:nsegy){
		if (yyy[j]>=quant2[k]){gy[j]=k}
	}
    }

    for (i in 1:nx){
	g=g_mult[,i]
	TSIstat=0
	for (iq in 1:nseg){
		for (ip in 1:nsegy){
			NE_pq=((length(yyy[(g==iq)]))*(length(yyy[(gy==ip)])))/(nr)
			class_term2=as.numeric(((length(yyy[(gy==ip)&(g==iq)])-NE_pq)^2)/(NE_pq))
			TSIstat=TSIstat+class_term2
		}
	}
	
	# sous Ho T suit un Chisq de  (nseg-1)(nsegy-1) d.d.l.
	SI.pvalue[i]=1-pchisq(as.numeric(TSIstat),(nseg -1)*(nsegy -1))
    }

# classement des parametres par ordre decroissant de sensibilite
# est le numero du parametre qui se trouve au ieme rang

    n_SI<-rep(0,nx)

    rSI<-rank(-abs(SI.pvalue),na.last=F, ties.method = "random")

    for (i in 1:nx){
	n_SI[nx+1-rSI[i]]=i
    }

    SI.pvalue<-format(SI.pvalue,digits=4)

  }
}

#**************************************
# Affichage complet
#**************************************

if (affiche == TRUE){

  write("*************************************************",file=FOUT)
  write(c("               Sortie ",nom[nx+i_y]),ncolumns=2,file=FOUT)
  write("*************************************************",file=FOUT)
  write(" ",file=FOUT)

  write("------------------------------------------------",file=FOUT)
  write("| --------------------------------------------- |",file=FOUT)
  write("| | Analyse de sensibilite des donnees brutes | |",file=FOUT)
  write("| --------------------------------------------- |",file=FOUT)
  write("-------------------------------------------------",file=FOUT)
  write(" ",file=FOUT)

#Affichage du R2

  write("--------------------------------",file=FOUT)
  write("| Coefficients de determination |",file=FOUT)
  write("--------------------------------",file=FOUT)
  write(" ",file=FOUT)
  write(c("R2 = ",R2,"     ;     R2 ajuste = ",R2aju),ncolumns=4,file=FOUT)
  write(" ",file=FOUT)

#Affichage des correlations lineaires (Pearson)

  write("-------------------------------------------------",file=FOUT)
  write("| Coefficients de correlation lineaire - Pearson |",file=FOUT)
  write("-------------------------------------------------",file=FOUT)
  write(" ",file=FOUT)

  write(c("       Pearson      rang"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
   write(c(nom[i],"   ",pear[i],"   ",nx+1-rpear[i]),ncolumns=5,file=FOUT)
  }
  write(" ",file=FOUT)

  write(c("rang  variable   Pearson    sensibilite=Pearson^2"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
    sens <- format(as.numeric(pear[n_pear[i]])^2,digits=4)
    write(c(i,"   ",nom[n_pear[i]],"     ",pear[n_pear[i]],"    ",sens),ncolumns=7,file=FOUT)
  }
  write(" ",file=FOUT)

#Affichage des coefficients de regression standard

  write("--------------------------------------------",file=FOUT)
  write("| Coefficients de regression standard - SRC |",file=FOUT)
  write("--------------------------------------------",file=FOUT)
  write(" ",file=FOUT)

  write(c("          SRC      rang"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
    write(c(nom[i],"   ",r[i],"   ",nx+1-rr[i]),ncolumns=5,file=FOUT)
  }
  write(" ",file=FOUT)

  write(c("rang  variable    SRC     sensibilite=SRC^2"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
    sens <- format(as.numeric(r[n_r[i]])^2,digits=4)
    write(c(i,"   ",nom[n_r[i]],"     ",r[n_r[i]],"     ",sens),ncolumns=7,file=FOUT)
  }
  write(" ",file=FOUT)

#Affichage des correlations partielles

  write("----------------------------------------------",file=FOUT)
  write("| Coefficients de correlation partielle - PCC |",file=FOUT)
  write("----------------------------------------------",file=FOUT)
  write(" ",file=FOUT)

  write(c("         PCC      rang"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
    write(c(nom[i],"   ",p[i],"   ",nx+1-rp[i]),ncolumns=5,file=FOUT)
  }
  write(" ",file=FOUT)

  write(c("rang  variable  PCC      sensibilite=PCC^2"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
    sens <- format(as.numeric(p[n_p[i]])^2,digits=4)
    write(c(i,"   ",nom[n_p[i]],"     ",p[n_p[i]],"     ",sens),ncolumns=7,file=FOUT)
  }
  write(" ",file=FOUT)

#Affichage du R2* (R2 des rangs)

  write("------------------------------------------",file=FOUT)
  write("| Coefficients de determination des rangs |",file=FOUT)
  write("------------------------------------------",file=FOUT)
  write(" ",file=FOUT)
  write(c("R2* = ",R2star,"     ;     R2* ajuste = ",R2staraju),ncolumns=4,file=FOUT)
  write(" ",file=FOUT)

#Affichage des correlations des rangs (Spearman)

  write("---------------------------------------------------",file=FOUT)
  write("| Coefficients de correlation des rangs - Spearman |",file=FOUT)
  write("---------------------------------------------------",file=FOUT)
  write(" ",file=FOUT)

  write(c("        Spearman   rang"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
   write(c(nom[i],"   ",spear[i],"   ",nx+1-rspear[i]),ncolumns=5,file=FOUT)
  }
  write(" ",file=FOUT)

  write(c("rang  variable   Spearman    sensibilite=Spearman^2"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
    sens <- format(as.numeric(spear[n_spear[i]])^2,digits=4)
    write(c(i,"   ",nom[n_spear[i]],"     ",spear[n_spear[i]],"    ",sens),ncolumns=7,file=FOUT)
  }
  write(" ",file=FOUT)

#Affichage des coefficients de regression des rangs

  write("-------------------------------------------------------",file=FOUT)
  write("| Coefficients de regression standard des rangs - SRRC |",file=FOUT)
  write("-------------------------------------------------------",file=FOUT)
  write(" ",file=FOUT)

  write(c("         SRRC      rang"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
    write(c(nom[i],"   ",u[i],"   ",nx+1-ru[i]),ncolumns=5,file=FOUT)
  }
  write(" ",file=FOUT)

  write(c("rang  variable    SRRC     sensibilite=SRRC^2"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
    sens <- format(as.numeric(u[n_u[i]])^2,digits=4)
    write(c(i,"   ",nom[n_u[i]],"     ",u[n_u[i]],"     ",sens),ncolumns=7,file=FOUT)
  }
  write(" ",file=FOUT)

#Affichage des correlations partielles des rangs

  write("---------------------------------------------------------",file=FOUT)
  write("| Coefficients de correlation partielle des rangs - PRCC |",file=FOUT)
  write("---------------------------------------------------------",file=FOUT)
  write(" ",file=FOUT)

  write(c("         PRCC      rang"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
    write(c(nom[i],"   ",q[i],"   ",nx+1-rq[i]),ncolumns=5,file=FOUT)
#    write(" ",file=FOUT)
  }
  write(" ",file=FOUT)

  write(c("rang  variable    PRCC     sensibilite=PRCC^2"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
    sens <- format(as.numeric(q[n_q[i]])^2,digits=4)
    write(c(i,"   ",nom[n_q[i]],"     ",q[n_q[i]],"     ",sens),ncolumns=7,file=FOUT)
#    write(" ",file=FOUT)
  }
  write(" ",file=FOUT)

#############################
#Affichage des indices bases sur les tests statistiques
  if (nseg>1){

#**************** CL Test ****************************************************
  write("---------------------------------------------------------",file=FOUT)
  write("| Indices des Common Locations - CL                     |",file=FOUT)
  write("---------------------------------------------------------",file=FOUT)
  write(" ",file=FOUT)
  write(c("Nombre de partitions = ",nseg),ncolumns=2,file=FOUT)
  write(" ",file=FOUT)

  write(c("         p-value      rang"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
    write(c(nom[i],"   ",CL.pvalue[i],"   ",nx+1-rkrus[i]),ncolumns=5,file=FOUT)
  }
  write(" ",file=FOUT)

  write(c("rang  variable    p-value"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
    write(c(i,"   ",nom[n_krus[i]],"     ",CL.pvalue[n_krus[i]]),ncolumns=5,file=FOUT)
  }
  write(" ",file=FOUT)

#**************** CMN Test ****************************************************
  write("---------------------------------------------------------",file=FOUT)
  write("| Indices des Common Means - CMN                     |",file=FOUT)
  write("---------------------------------------------------------",file=FOUT)
  write(" ",file=FOUT)
  write(c("Nombre de partitions = ",nseg),ncolumns=2,file=FOUT)
  write(" ",file=FOUT)

  write(c("         p-value      rang"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
    write(c(nom[i],"   ",CMN.pvalue[i],"   ",nx+1-rCMN[i]),ncolumns=5,file=FOUT)
  }
  write(" ",file=FOUT)

  write(c("rang  variable    p-value"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
    write(c(i,"   ",nom[n_CMN[i]],"     ",CMN.pvalue[n_CMN[i]]),ncolumns=5,file=FOUT)
  }
  write(" ",file=FOUT)

#**************** CMD Test ****************************************************
  write("---------------------------------------------------------",file=FOUT)
  write("| Indices des Common Medians - CMD                     |",file=FOUT)
  write("---------------------------------------------------------",file=FOUT)
  write(" ",file=FOUT)
  write(c("Nombre de partitions = ",nseg),ncolumns=2,file=FOUT)
  write(" ",file=FOUT)

  write(c("         p-value      rang"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
    write(c(nom[i],"   ",CMD.pvalue[i],"   ",nx+1-rCMD[i]),ncolumns=5,file=FOUT)
  }
  write(" ",file=FOUT)

  write(c("rang  variable    p-value"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
    write(c(i,"   ",nom[n_CMD[i]],"     ",CMD.pvalue[n_CMD[i]]),ncolumns=5,file=FOUT)
  }
  write(" ",file=FOUT)

#**************** CV Test ****************************************************
  write("---------------------------------------------------------",file=FOUT)
  write("| Indices des Common Variances - CV                     |",file=FOUT)
  write("---------------------------------------------------------",file=FOUT)
  write(" ",file=FOUT)
  write(c("Nombre de partitions = ",nseg),ncolumns=2,file=FOUT)
  write(" ",file=FOUT)

  write(c("         p-value      rang"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
    write(c(nom[i],"   ",CV.pvalue[i],"   ",nx+1-rCV[i]),ncolumns=5,file=FOUT)
  }
  write(" ",file=FOUT)

  write(c("rang  variable    p-value"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
    write(c(i,"   ",nom[n_CV[i]],"     ",CV.pvalue[n_CV[i]]),ncolumns=5,file=FOUT)
  }
  write(" ",file=FOUT)

#**************** SI Test ****************************************************
  if (nsegy > 1){
    write("---------------------------------------------------------",file=FOUT)
    write("| Indices Statistical Independance - SI                 |",file=FOUT)
    write("---------------------------------------------------------",file=FOUT)
    write(" ",file=FOUT)
    write(c("Nombre de partitions = ",nseg,"/",nsegy),ncolumns=4,file=FOUT)
    write(" ",file=FOUT)

    write(c("         p-value      rang"),file=FOUT)
    write(" ",file=FOUT)

    for(i in 1:nx){
      write(c(nom[i],"   ",SI.pvalue[i],"   ",nx+1-rSI[i]),ncolumns=5,file=FOUT)
    }
    write(" ",file=FOUT)

    write(c("rang  variable    p-value"),file=FOUT)
    write(" ",file=FOUT)

    for(i in 1:nx){
      write(c(i,"   ",nom[n_SI[i]],"     ",SI.pvalue[n_SI[i]]),ncolumns=5,file=FOUT)
    }
    write(" ",file=FOUT)
  }}

#********************************************

  write("------------------------------------------------------------",file=FOUT)
  write("| -------------------------------------------------------- |",file=FOUT)
  write("| | Analyse de sensibilite des donnees centrees-reduites | |",file=FOUT)
  write("| -------------------------------------------------------- |",file=FOUT)
  write("------------------------------------------------------------",file=FOUT)
  write(" ",file=FOUT)

#Affichage du R2

  write("--------------------------------",file=FOUT)
  write("| Coefficients de determination |",file=FOUT)
  write("--------------------------------",file=FOUT)
  write(" ",file=FOUT)
  write(c("R2 = ",R2m,"     ;     R2 ajuste = ",R2ajum),ncolumns=4,file=FOUT)
  write(" ",file=FOUT)

#Affichage des coefficients de regression standard

  write("--------------------------------------------",file=FOUT)
  write("| Coefficients de regression standard - SRC |",file=FOUT)
  write("--------------------------------------------",file=FOUT)
  write(" ",file=FOUT)

  write(c("          SRC      rang"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
    write(c(nom[i],"   ",rm[i],"   ",nx+1-rrm[i]),ncolumns=5,file=FOUT)
  }
  write(" ",file=FOUT)

  write(c("rang  variable    SRC     sensibilite=SRC^2"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
    sens <- format(as.numeric(rm[n_rm[i]])^2,digits=4)
    write(c(i,"   ",nom[n_rm[i]],"     ",rm[n_rm[i]],"     ",sens),ncolumns=7,file=FOUT)
  }
  write(" ",file=FOUT)

#Affichage des correlations partielles

  write("----------------------------------------------",file=FOUT)
  write("| Coefficients de correlation partielle - PCC |",file=FOUT)
  write("----------------------------------------------",file=FOUT)
  write(" ",file=FOUT)

  write(c("         PCC      rang"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
    write(c(nom[i],"   ",pm[i],"   ",nx+1-rpm[i]),ncolumns=5,file=FOUT)
  }
  write(" ",file=FOUT)

  write(c("rang  variable  PCC      sensibilite=PCC^2"),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:nx){
    sens <- format(as.numeric(pm[n_pm[i]])^2,digits=4)
    write(c(i,"   ",nom[n_pm[i]],"     ",pm[n_pm[i]],"     ",sens),ncolumns=7,file=FOUT)
  }
  write(" ",file=FOUT)
  }

  # Affichage resume
  else{
    R2_n[i_y] <- R2
    R2aju_n[i_y] <- R2aju
    pear_n[,i_y] <- pear
    r_n[,i_y] <- r
    p_n[,i_y] <- p
    R2star_n[i_y] <- R2star
    R2staraju_n[i_y] <- R2staraju
    spear_n[,i_y] <- spear
    u_n[,i_y] <- u
    q_n[,i_y] <- q
    if (nseg>1){
    CL_n[,i_y] <- CL.pvalue
    CMN_n[,i_y] <- CMN.pvalue
    CMD_n[,i_y] <- CMD.pvalue
    CV_n[,i_y] <- CV.pvalue
    SI_n[,i_y] <- SI.pvalue}
    }
}
# FIN de la BOUCLE sur les SORTIES

#************************************************************************
# Affichage resume (pas de tri et pas centre-reduit)
#************************************************************************
if (affiche == FALSE){

  write("--------------------------------",file=FOUT)
  write("| Coefficients de determination |",file=FOUT)
  write("--------------------------------",file=FOUT)
  write(" ",file=FOUT)
  write.table(array(c("",dimnames(pear_n)[[2]]),dim=c(1,ny+1)),file=FOUT,sep="\t",quote=FALSE,row.names=FALSE,col.names=FALSE)
  write(c("R2\t",R2_n),ncolumns=1+length(R2_n),file=FOUT)
  write(c("R2aju\t",R2aju_n),ncolumns=1+length(R2aju_n),file=FOUT)
  write(" ",file=FOUT)
  
  R2_n <<- R2_n
  R2aju_n <<- R2aju_n
  
  write("-------------------------------------------------",file=FOUT)
  write("| Coefficients de correlation lineaire - Pearson |",file=FOUT)
  write("-------------------------------------------------",file=FOUT)
  write(" ",file=FOUT)
  write.table(array(c("",dimnames(pear_n)[[2]]),dim=c(1,ny+1)),file=FOUT,sep="\t",quote=F,row.names=F,col.names=F)
  write.table(pear_n,quote=F,col.names=F,sep=" ",file=FOUT)
  write(" ",file=FOUT)
  
  pear_n <<- pear_n
  
  write("--------------------------------------------",file=FOUT)
  write("| Coefficients de regression standard - SRC |",file=FOUT)
  write("--------------------------------------------",file=FOUT)
  write(" ",file=FOUT)
  write.table(array(c("",dimnames(r_n)[[2]]),dim=c(1,ny+1)),file=FOUT,sep="\t",quote=F,row.names=F,col.names=F)
  write.table(r_n,quote=F,col.names=F,sep=" ",file=FOUT)
  write(" ",file=FOUT)
  
  src_n <<- r_n

  write("----------------------------------------------",file=FOUT)
  write("| Coefficients de correlation partielle - PCC |",file=FOUT)
  write("----------------------------------------------",file=FOUT)
  write(" ",file=FOUT)
  write.table(array(c("",dimnames(p_n)[[2]]),dim=c(1,ny+1)),file=FOUT,sep="\t",quote=F,row.names=F,col.names=F)
  write.table(p_n,quote=F,col.names=F,sep=" ",file=FOUT)
  write(" ",file=FOUT)
  
  pcc_n <<- p_n
  
  write("------------------------------------------",file=FOUT)
  write("| Coefficients de determination des rangs |",file=FOUT)
  write("------------------------------------------",file=FOUT)
  write(" ",file=FOUT)
  write.table(array(c("",dimnames(spear_n)[[2]]),dim=c(1,ny+1)),file=FOUT,sep="\t",quote=FALSE,row.names=FALSE,col.names=FALSE)
  write(c("R2*\t",R2star_n),ncolumns=1+length(R2star_n),file=FOUT)
  write(c("R2*aju\t",R2staraju_n),ncolumns=1+length(R2staraju_n),file=FOUT)
  write(" ",file=FOUT)
  
  R2star_n <<- R2star_n
  R2staraju_n <<- R2staraju_n
  
  write("---------------------------------------------------",file=FOUT)
  write("| Coefficients de correlation des rangs - Spearman |",file=FOUT)
  write("---------------------------------------------------",file=FOUT)
  write(" ",file=FOUT)
  write.table(array(c("",dimnames(spear_n)[[2]]),dim=c(1,ny+1)),file=FOUT,sep="\t",quote=F,row.names=F,col.names=F)
  write.table(spear_n,quote=F,col.names=F,sep=" ",file=FOUT)
  write(" ",file=FOUT)
  
  spear_n <<- spear_n

  write("-------------------------------------------------------",file=FOUT)
  write("| Coefficients de regression standard des rangs - SRRC |",file=FOUT)
  write("-------------------------------------------------------",file=FOUT)
  write(" ",file=FOUT)
  write.table(array(c("",dimnames(u_n)[[2]]),dim=c(1,ny+1)),file=FOUT,sep="\t",quote=F,row.names=F,col.names=F)
  write.table(u_n,quote=F,col.names=F,sep=" ",file=FOUT)
  write(" ",file=FOUT)
  
  srrc_n <<- u_n

  write("---------------------------------------------------------",file=FOUT)
  write("| Coefficients de correlation partielle des rangs - PRCC |",file=FOUT)
  write("---------------------------------------------------------",file=FOUT)
  write(" ",file=FOUT)
  write.table(array(c("",dimnames(q_n)[[2]]),dim=c(1,ny+1)),file=FOUT,sep="\t",quote=F,row.names=F,col.names=F)
  write.table(q_n,quote=F,col.names=F,sep=" ",file=FOUT)
  write(" ",file=FOUT)
  
  prcc_n <<- q_n

  if (nseg>1){
  write("---------------------------------------------------------",file=FOUT)
  write("|  Indices des Common Locations - CL (p-values) |",file=FOUT)
  write("---------------------------------------------------------",file=FOUT)
  write(" ",file=FOUT)
  write.table(array(c("",dimnames(CL_n)[[2]]),dim=c(1,ny+1)),file=FOUT,sep="\t",quote=F,row.names=F,col.names=F)
  write.table(CL_n,quote=F,col.names=F,sep=" ",file=FOUT)
  write(" ",file=FOUT)

  kruskal_n <<- CL_n


  write("---------------------------------------------------------",file=FOUT)
  write("|  Indices des Common Means - CMN (p-values) |",file=FOUT)
  write("---------------------------------------------------------",file=FOUT)
  write(" ",file=FOUT)
  write.table(array(c("",dimnames(CMN_n)[[2]]),dim=c(1,ny+1)),file=FOUT,sep="\t",quote=F,row.names=F,col.names=F)
  write.table(CMN_n,quote=F,col.names=F,sep=" ",file=FOUT)
  write(" ",file=FOUT)

  CommonMeans_n <<- CMN_n

  write("---------------------------------------------------------",file=FOUT)
  write("|  Indices des Common Medians - CMD (p-values) |",file=FOUT)
  write("---------------------------------------------------------",file=FOUT)
  write(" ",file=FOUT)
  write.table(array(c("",dimnames(CMD_n)[[2]]),dim=c(1,ny+1)),file=FOUT,sep="\t",quote=F,row.names=F,col.names=F)
  write.table(CMD_n,quote=F,col.names=F,sep=" ",file=FOUT)
  write(" ",file=FOUT)

  CommonMed_n <<- CMD_n

  write("---------------------------------------------------------",file=FOUT)
  write("|  Indices des Common Variances - CV (p-values) |",file=FOUT)
  write("---------------------------------------------------------",file=FOUT)
  write(" ",file=FOUT)
  write.table(array(c("",dimnames(CV_n)[[2]]),dim=c(1,ny+1)),file=FOUT,sep="\t",quote=F,row.names=F,col.names=F)
  write.table(CV_n,quote=F,col.names=F,sep=" ",file=FOUT)
  write(" ",file=FOUT)

  CommonVar_n <<- CV_n

  if (nsegy>1){
  write("---------------------------------------------------------",file=FOUT)
  write("|  Indices des Statistical Independances - SI (p-values) |",file=FOUT)
  write("---------------------------------------------------------",file=FOUT)
  write(" ",file=FOUT)
  write.table(array(c("",dimnames(SI_n)[[2]]),dim=c(1,ny+1)),file=FOUT,sep="\t",quote=F,row.names=F,col.names=F)
  write.table(SI_n,quote=F,col.names=F,sep=" ",file=FOUT)
  write(" ",file=FOUT)

  StatisticalInd_n <<- SI_n
  }}}

#Fermeture du fichier de resultats
close(FOUT)

# Affichage du fichier de resultat
if (Sys.getenv("OS")!="Windows_NT"){ pager_OS="cat"}
else{ pager_OS=""}

file.show("res-SENSIBILITE.out",pager=pager_OS)

}
