sobolS<-function(f=a,modele=b,methode=0,dim_x,x=NULL,
                logx=rep(FALSE,dim_x),logy=FALSE,S=10000,R=1,Sij=TRUE,
                LHS=FALSE,lois=rep(0,dim_x),
                paramlois=array(0,dim=c(4,dim_x)),
                tronq=rep(F,dim_x),paramtronq=array(0,dim=c(2,dim_x)),...){

#appel : sobolS(f=hs,dim_x=3,S=10000,R=1,LHS=TRUE,
#           paramlois=array(c(-pi,pi,0,0,-pi,pi,0,0,-pi,pi,0,0),dim=c(4,3)))

#---------------------------------------------------------------------------|
#Fonction pour calculer les indices de sensibilite de Sobol                 |
#---------------------------------------------------------------------------|
#Arguments obligatoires dans le cas d'un modele sous forme d'objet R :      |
#       modele : objet contenant le modele                                  |
#       methode : numero de la methode, 0 si fonction utilisateur           |
#         1=GLM, 2=PLS, 3=SVM, 4=BOOSTING, 5=RF, 6=RN, 7=GAM, 8=PG, 9=PGdice|
#       dim_x : nombre de parametres d'entree dans le modele                |
#                                                                           |
#Arguments obligatoires dans le cas d'une fonction :                        |
#       f : objet representant la fonction (prealablement declaree)         |
#       dim_x : nombre de parametres d'entree dans le modele                |
#                                                                           |
#Arguments facultatifs :                                                    |
#             x : matrice de variables d'entree (pour calculer les bornes   |
#               de la loi uniforme si necessaire)                           |
#             logx : vecteur logx=c(TRUE,TRUE,...,TRUE), correspondant aux  |
#               parametres d'entree transformees en log lors de             |
#               l'ajustement du modele                                      |
#             logy : mettre logy=TRUE si la variable de sortie a ete        |
#               transformee en log lors de l'ajustement du modele           |
#             S : nombre de simulations                                     |
#             R : nombre de repetitions du programme                        |
#             Sij : TRUE pour calculer les indices d'ordre 2                |
#             LHS : TRUE pour utiliser un echantillonnage LHS lors du       |
#               calcul des indices de Sobol                                 |
#               Par defaut : FALSE pour un echantillonnage simple           |
#             lois : vecteur contenant les types de distribution de proba   |
#               pour chaque entree                                          |
#               0=uniforme ; 1=normale ; 2=lognormale ; 3=weibull           | 
#               4=exponentielle ; 5=beta ; 6=triangulaire ; 7=trapezoidale  |
#               10=gumbel                                                   |
#               Par defaut, on prend la loi uniforme                        |
#             paramlois : tableau avec les parametres de chaque loi (max=4) |
#               pour chaque entree (range par colonne) :                    |
#               (min,max,0,0) pour uniforme (par dfaut : min=0, max=1)     |
#               (moy,ecart-type,0,0) pour normale,                          |
#               (moy du log, ecart-type du log,0,0) pour lognormale,        |
#               (forme,echelle,0,0) pour Weibull,                           |
#               (lambda,0,0,0) pour exponentielle,                          |
#               (shape1, shape2,0,0) pour beta,                             |
#               (min,mode,max,0) pour triangulaire,                         |
#               (min,mode1,mode2,max) pour trapezoidale,                    |
#               (mode,echelle,0,0) pour Gumbel                              |
#                Par defaut, on a la loi uniforme avec (min,max) calcules a |
#                partir des parametres d'entree x                           |
#             tronq : vecteur pour selectionner ou non une loi tronquee     |
#               TRUE pour loi tronquee, FALSE sinon (par dfaut)            |
#             paramtronq : tableau avec les parametres de troncature de     |
#               chaque loi : (min,max) range par colonne                    |
#---------------------------------------------------------------------------|
#Sortie : les indices Si, Sij et STi a la fin du fichier "res-SOBOL.out"    |
#         (fichier non cras)                                              |
#         Pour chaque indice, on donne son estimation moyenne, minimale,    |
#         maximale, ses ecart-types empirique et asymptotique (issu du TCL) |
#---------------------------------------------------------------------------|
#      Auteur : B. Iooss et A. Ducellier                                    |
#---------------------------------------------------------------------------|

#Initialisation des vecteurs contenant les indices de sensibilite

S1<-array(0,dim=c(R,dim_x))
S2<-array(0,dim=c(R,dim_x,dim_x))
St<-array(0,dim=c(R,dim_x))

et1 <- rep(0,dim_x) # ecart-type asymptotique
et2<-matrix(0,nr=dim_x,nc=dim_x) # ecart-type asymptotique
ett<-rep(0,dim_x) # ecart-type asymptotique

# Pour definir les bornes de la loi uniforme si necessaire
for(i in 1:dim_x){
  if (lois[i]==0 & paramlois[1,i]==0 & paramlois[2,i]==0){
    if (logx[i]==FALSE){
      paramlois[1,i]<-min(x[,i])
      paramlois[2,i]<-max(x[,i])}
    else{
      paramlois[1,i]<-exp(min(x[,i]))
      paramlois[2,i]<-exp(max(x[,i]))}}}

#--------
#Boucle |
#--------

for(k in 1:R){

#Tirage des echantillons
  if (LHS==FALSE){
    X1<-samplingSimple(dim_x,N=S,lois=lois,paramlois=paramlois)
    X2<-samplingSimple(dim_x,N=S,lois=lois,paramlois=paramlois)}
  else{
    X1<-samplingLHS(dim_x,N=S,lois=lois,paramlois=paramlois)
    X2<-samplingLHS(dim_x,N=S,lois=lois,paramlois=paramlois)}

  for(i in 1:dim_x){ if (logx[i]==TRUE){ X1[,i]=log(X1[,i]); X2[,i]=log(X2[,i])}}

#---------------------------
#Calcul de la moyenne de y |
#---------------------------

  if (methode>0) pred<-predictSR(modele,X1,methode)
  else pred<-f(X1,...)
  if (logy) pred=exp(pred)
  my<-mean(pred)

#-------------
#Calcul de V0 |
#-------------
  V0<-var(pred)

#---------------
#Calcul des Vi |
#---------------
  V<-rep(0,dim_x)

  for(i in 1:dim_x){

    pred1<-pred

    X2b<-X2 
    X2b[,i]<-X1[,i]

    if (methode>0) pred2<-predictSR(modele,X2b,methode)
    else pred2<-f(X2b,...)

    if (logy==TRUE){ pred2=exp(pred2)}

    V[i]<-((1/S)*sum(pred1*pred2))-my^2
    et1[i] <- et1[i] + sd(pred1*pred2) / V0 / sqrt(S)
  }

#----------------
#Calcul des Vij |
#----------------
  V2<-matrix(0,nr=dim_x,nc=dim_x)

  if (Sij==TRUE){
    for(i in 1:(dim_x-1)){
      for(j in (i+1):dim_x){

        pred1<-pred

        X2b<-X2
        X2b[,i]<-X1[,i]
        X2b[,j]<-X1[,j]

        if (methode>0) pred2<-predictSR(modele,X2b,methode)
        else pred2<-f(X2b,...)

        if (logy==TRUE){ pred2=exp(pred2)}

        V2[i,j]<-((1/S)*sum(pred1*pred2))-V[i]-V[j]-my^2
	et2[i,j] <- et2[i,j] + sd(pred1*pred2) / V0 / sqrt(S)
  }}}

#----------------
#Calcul des Vti |
#----------------
  Vt<-rep(0,dim_x)

  for(i in 1:dim_x){
    
    pred1<-pred

    X1b<-X1
    X1b[,i]<-X2[,i]

    if (methode>0) pred2<-predictSR(modele,X1b,methode)
    else pred2<-f(X1b,...)

    if (logy==TRUE){ pred2=exp(pred2)}

    Vt[i]<-((1/S)*sum(pred1*pred2))-my^2
    ett[i] <- ett[i] + sd(pred1*pred2) / V0 / sqrt(S)
  }

#-----------------------------
#Calcul des indices de Sobol |
#-----------------------------
  S1[k,]<-V/V0
  St[k,]<-1-(Vt/V0)
  for (i in 1:dim_x) S2[k,i,]<-V2[i,]/V0

}
#----------------------
#Fin de la boucle sur R|
#----------------------

#Calcul des moyennes, min, max et ecart-types

#Initialisation des vecteurs
mS1<-rep(0,dim_x)
mS2<-matrix(0,nr=dim_x,nc=dim_x)
mSt<-rep(0,dim_x)
minS1<-rep(0,dim_x)
minS2<-matrix(0,nr=dim_x,nc=dim_x)
minSt<-rep(0,dim_x)
maxS1<-rep(0,dim_x)
maxS2<-matrix(0,nr=dim_x,nc=dim_x)
maxSt<-rep(0,dim_x)
sigS1<-rep(0,dim_x)
sigS2<-matrix(0,nr=dim_x,nc=dim_x)
sigSt<-rep(0,dim_x)
met1<-rep(0,dim_x)
met2<-matrix(0,nr=dim_x,nc=dim_x)
mett<-rep(0,dim_x)

for(i in 1:dim_x){
  mS1[i]<-mean(S1[,i])
  minS1[i]<-min(S1[,i])
  maxS1[i]<-max(S1[,i])
  sigS1[i]<-sqrt(var(S1[,i]))
  met1[i]<-et1[i]/R
}

if (Sij==T){
  for(i in 1:(dim_x-1)){
    for(j in (i+1):dim_x){
      mS2[i,j]<-mean(S2[,i,j])
      minS2[i,j]<-min(S2[,i,j])
      maxS2[i,j]<-max(S2[,i,j])
      sigS2[i,j]<-sqrt(var(S2[,i,j]))
      met2[i,j]<-et2[i,j]/R
    }
  }
}

for(i in 1:dim_x){
  mSt[i]<-mean(St[,i])
  minSt[i]<-min(St[,i])
  maxSt[i]<-max(St[,i])
  sigSt[i]<-sqrt(var(St[,i]))
  mett[i]<-ett[i]/R
}

#-----------------------
#Affichage dans fichier |
#-----------------------

#Preparation des donnees pour l'affichage
mS1<-format(mS1,digits=3)
mS2<-format(mS2,digits=3)
mSt<-format(mSt,digits=3)
minS1<-format(minS1,digits=3)
minS2<-format(minS2,digits=3)
minSt<-format(minSt,digits=3)
maxS1<-format(maxS1,digits=3)
maxS2<-format(maxS2,digits=3)
maxSt<-format(maxSt,digits=3)
sigS1<-format(sigS1,digits=3)
sigS2<-format(sigS2,digits=3)
sigSt<-format(sigSt,digits=3)
met1<-format(met1,digits=3)
met2<-format(met2,digits=3)
mett<-format(mett,digits=3)

FSOB <- file("res-SOBOL.out","a+")
write("--------------------------------------------------",file=FSOB)

if (methode==1){
  write("Modele GLM : Indices de sensibilite de Sobol",file=FSOB)
  print("Sobol GLM termine");}
if (methode==2){
  write("Modele PLS : Indices de sensibilite de Sobol",file=FSOB)
  print("Sobol PLS termine");}
if (methode==3){
  write("Modele SVM : Indices de sensibilite de Sobol",file=FSOB)
  print("Sobol SVM termine");}
if (methode==4){
  write("Modele BOOSTING : Indices de sensibilite de Sobol",file=FSOB)
  print("Sobol BOOSTING termine");}
if (methode==5){
  write("Modele FORETS ALEATOIRES : Indices de sensibilite de Sobol",file=FSOB)
  print("Sobol FORETS ALEATOIRES termine");}
if (methode==6){
  write("Modele RESEAU de NEURONES : Indices de sensibilite de Sobol",file=FSOB)
  print("Sobol RESEAU de NEURONES termine");}
if (methode==7){
  write("Modele GAM : Indices de sensibilite de Sobol",file=FSOB)
  print("Sobol GAM termine");}
if (methode==8){
  write("Modele PG : Indices de sensibilite de Sobol",file=FSOB)
  print("Sobol PG termine");}
if (methode==9){
  write("Modele PGdice : Indices de sensibilite de Sobol",file=FSOB)
  print("Sobol PGdice termine");}

write(" ",file=FSOB)

write("------------------",file=FSOB)
write("| Les indices Si |",file=FSOB)
write("------------------",file=FSOB)

for(i in 1:dim_x){ write(c("S",i," : Moy=",mS1[i]," Min=",minS1[i]," Max=",maxS1[i]," E-t=",sigS1[i]," E-t(TCL)=",met1[i]),ncolumns=12,file=FSOB)}
write(" ",file=FSOB)

if (Sij==TRUE){
  write("-------------------",file=FSOB)
  write("| Les indices Sij |",file=FSOB)
  write("-------------------",file=FSOB)

  for(i in 1:(dim_x-1)){
    for(j in (i+1):dim_x){
      write(c("S",i,j," : Moy=",mS2[i,j]," Min=",minS2[i,j]," Max=",maxS2[i,j]," E-t=",sigS2[i,j]," E-t(TCL)=",met2[i,j]),ncolumns=13,file=FSOB)}}
  write(" ",file=FSOB)}

write("-------------------",file=FSOB)
write("| Les indices Sti |",file=FSOB)
write("-------------------",file=FSOB)

for(i in 1:dim_x){ write(c("St",i," : Moy=",mSt[i]," Min=",minSt[i]," Max=",maxSt[i]," E-t=",sigSt[i]," E-t(TCL)=",mett[i]),ncolumns=12,file=FSOB)}
write(" ",file=FSOB)

close(FSOB)
}
