statsconverg<-function(x,nom=c(NA),cvplot=T,nrepet=1,nboot=100,fboot=4,
                       kboot=1,conf=0.95,moments=FALSE,ampl=2){

#-----------------------------------------------------------------------
#Etude de la representativite des donnees en analysant la convergence  |
#des moyennes, ecart-types, skewness et kurtosis des colonnes de x,    |
#avec des intervalles de confiance donnes par bootstrap                |
#Etude de la convergence de la somme des accroissements des moments et |
#de la somme des accroissements des IC des moments                     |
#----------------------------------------------------------------------|
#Arguments :                                                           |
#                                                                      |
#obligatoire : x : matrice des variables d'entree (format colonnes)    |
#                                                                      |
#facultatif : nom    : vecteur des noms de variable (7 caracteres max) |
#                     (par defaut : header du tableau x)               |
#             cvplot : TRUE -> tous les graphes de convergence         |
#                      FALSE -> seulement la cvence des accroissements |
#             nrepet : nombre de repetitions de graphes de convergence |
#             nboot  : nombre de repliques bootstrap                   |
#             fboot  : 1ere taille d'echantillon pour la convergence   |
#             kboot  : increment de la taille d'echantillon            |
#             conf   : niveau de confiance pour l'intervalle bootstrap |
#             moments: TRUE pour avoir les skewness et kurtosis        |
#                      en plus des moyennes et ecart-type              |
#                      (ces calculs ne fonctionnent que pour 1 colonne)|
#             ampl   : facteur d'amplification de l'axe des ordonnees  |
#                      pour graphes de cvence des sommes des accroiss  |
#----------------------------------------------------------------------|
#Sortie : Graphes de convergence                                       |
#----------------------------------------------------------------------|
#Utilisation :                                                         |
#  -Au prealable, il faut allouer x sous R                             |
#    Exemple : mat<-read.table(file="MOX_DEI.txt",header=TRUE,dec=",") |
#            x<-mat[,2:length(mat)]                                    |
#            x<-x+runif(dim(x)[[1]]*dim(x)[[2]],min=-0.0005,max=0.0005)|
#            x<-read.table(file="nom_fichier")                         |
#                                                                      |
#  - Il faut le meme nombre de donnees dans chaque colonne             |
#----------------------------------------------------------------------|
#      Auteur : B. Iooss                                               |
#----------------------------------------------------------------------|
  
# *********************************************************************
# FONCTION UTILISEE :
#      bootstats.R (intervalle de confiance bootstrap)
# *********************************************************************
# LIBRAIRIES REQUISES
  library(boot)
  library(e1071) # A INSTALLER
  
# *********************************************************************
# INITIALISATIONS

# Transformation de x en matrice
  if (is.matrix(x)==FALSE & is.vector(x)==TRUE) x<-matrix(x,ncol=1)
  
# Cacul des dimensions de x
  n<-dim(x)[1]
  np<-dim(x)[2]

# Noms des variables
  for(i in 1:np){
    if (is.na(nom[i])==T){
      if (is.null(dimnames(x)[[2]][i])==F) nom[i]<-dimnames(x)[[2]][i]
      else nom[i] <- ""}}
#  x<-matrix(c(x),nr=n,nc=np,dimnames=list(1:n,nom[1:np]))

# Verification des arguments
  if (nboot <= 1) {
    print("nboot doit etre > 1, on prendra nboot=100")
    nboot <- 100}
  if (fboot >= n) {
    print("fboot doit etre < nb de donnees, on prendra fboot=1")
    fboot <- 1}
  if (np > 1 & moments==TRUE){
    print("On ne peut pas calculer les skewness et kurtosis quand on a plusieurs colonnes dans x")
    moments <- FALSE}

# Initialisation de vecteurs pour les graphiques
    devcv <- rep(0,np)
    devcvIC <- rep(0,np)

# *********************************************************************

# BOUCLE sur les REPETITIONS de l'ETUDE de la CONVERGENCE
  for (jj in 1:nrepet){
    
# Initialisation des matrices
    xx <- matrix(0,nr=n,nc=np)
    moy <- matrix(0,nr=np,nc=5)
    moymean <- matrix(0,nr=n,nc=np)
    moybias <- matrix(0,nr=n,nc=np)
    moycimin <- matrix(0,nr=n,nc=np)
    moycimax <- matrix(0,nr=n,nc=np)
    moyinc <- matrix(0,nr=n,nc=np)
    moyICinc <- matrix(0,nr=n,nc=np)
    ect <- matrix(0,nr=np,nc=5)
    ectmean <- matrix(0,nr=n,nc=np)
    ectbias <- matrix(0,nr=n,nc=np)
    ectcimin <- matrix(0,nr=n,nc=np)
    ectcimax <- matrix(0,nr=n,nc=np)
    ectinc <- matrix(0,nr=n,nc=np)
    ectICinc <- matrix(0,nr=n,nc=np)
    if (moments==TRUE){
      skew <- matrix(0,nr=np,nc=5)
      skewmean <- matrix(0,nr=n,nc=np)
      skewbias <- matrix(0,nr=n,nc=np)
      skewcimin <- matrix(0,nr=n,nc=np)
      skewcimax <- matrix(0,nr=n,nc=np)
      skewinc <- matrix(0,nr=n,nc=np)
      skewICinc <- matrix(0,nr=n,nc=np)
      kurt <- matrix(0,nr=np,nc=5)
      kurtmean <- matrix(0,nr=n,nc=np)
      kurtbias <- matrix(0,nr=n,nc=np)
      kurtcimin <- matrix(0,nr=n,nc=np)
      kurtcimax <- matrix(0,nr=n,nc=np)
      kurtinc <- matrix(0,nr=n,nc=np)
      kurtICinc <- matrix(0,nr=n,nc=np)}
  
# Reechantillonnage de la base initiale
    for (j in 1:np) xx[,j] <- sample(x[,j],n)

# BOUCLE sur la TAILLE d'ECHANTILLON
    i <- fboot
    index <- NULL
    while (i <= n){

# Bootstrap sur la moyenne
      bboot <- boot(xx[1:i,],function(s,j){mean(as.data.frame(s)[j,])},R=nboot)
      moy <- bootstats(bboot,conf,"basic")[,1:5]
      moymean[i,] <- moy[,1] # Moyenne echantillon original
      moybias[i,] <- moy[,2] + moy[,1] # estimateur bootstrap
      moycimin[i,] <- moy[,4] # Intervalle de confiance
      moycimax[i,] <- moy[,5]

      if (i<fboot+kboot){ # convergence de la somme des accroissements
	moyinc[i,] <- 0 
	moyICinc[i,] <- 0 }
      else{
        moyinc[i,] <- moyinc[i-kboot,]+abs(moymean[i,]-moymean[i-kboot,])
	moyICinc[i,] <- moyICinc[i-kboot,]+abs(moycimax[i,]-moycimin[i,]-moycimax[i-kboot,]+moycimin[i-kboot,])}

# Bootstrap sur l'ecart-type
      bboot <- boot(xx[1:i,],function(s,j){sd(as.data.frame(s)[j,])},R=nboot)
      ect <- bootstats(bboot,conf,"basic")[,1:5]
      ectmean[i,] <- ect[,1] # ecart-type echantillon original
      ectbias[i,] <- ect[,2] + ect[,1] # estimateur bootstrap
      ectcimin[i,] <- ect[,4] # Intervalle de confiance
      ectcimax[i,] <- ect[,5]

      if (i<fboot+kboot){ # convergence de la somme des accroissements
	ectinc[i,] <- 0 
	ectICinc[i,] <- 0 }
      else{
        ectinc[i,] <- ectinc[i-kboot,]+abs(ectmean[i,]-ectmean[i-kboot,])
	ectICinc[i,] <- ectICinc[i-kboot,]+abs(ectcimax[i,]-ectcimin[i,]-ectcimax[i-kboot,]+ectcimin[i-kboot,])}

      if (moments==TRUE){
# Bootstrap sur le skewness
        bboot <- boot(xx[1:i,],function(s,j){skewness(as.data.frame(s)[j,],na.rm=T)},R=nboot)
        skew <- bootstats(bboot,conf,"basic")[,1:5]
        skewmean[i,] <- skew[,1] # Skewness echantillon original
        skewbias[i,] <- skew[,2] + skew[,1] # estimateur bootstrap
        skewcimin[i,] <- skew[,4] # Intervalle de confiance
        skewcimax[i,] <- skew[,5]

        if (i<fboot+kboot){ # convergence de la somme des accroissements
	  skewinc[i,] <- 0 
	  skewICinc[i,] <- 0 }
        else{
          skewinc[i,] <- skewinc[i-kboot,]+abs(skewmean[i,]-skewmean[i-kboot,])
	  skewICinc[i,] <- skewICinc[i-kboot,]+abs(skewcimax[i,]-skewcimin[i,]-skewcimax[i-kboot,]+skewcimin[i-kboot,])}

# Bootstrap sur le kurtosis
        bboot <- boot(xx[1:i,],function(s,j){kurtosis(as.data.frame(s)[j,],na.rm=T)},R=nboot)
        kurt <- bootstats(bboot,conf,"basic")[,1:5]
        kurtmean[i,] <- kurt[,1] # Kurtosis echantillon original
        kurtbias[i,] <- kurt[,2] + kurt[,1] # estimateur bootstrap
        kurtcimin[i,] <- kurt[,4] # Intervalle de confiance
        kurtcimax[i,] <- kurt[,5]

        if (i<fboot+kboot){ # convergence de la somme des accroissements
	  kurtinc[i,] <- 0 
	  kurtICinc[i,] <- 0 }
        else{
          kurtinc[i,] <- kurtinc[i-kboot,]+abs(kurtmean[i,]-kurtmean[i-kboot,])
  	  kurtICinc[i,] <- kurtICinc[i-kboot,]+abs(kurtcimax[i,]-kurtcimin[i,]-kurtcimax[i-kboot,]+kurtcimin[i-kboot,])}
      }

      index <- c(index,i)
      i <- i+kboot
    }

# *********************************************************************
# GRAPHIQUES
    nindex <- length(index)
    for (j in 1:np){

#**********************************************
# convergence des moments et des IC des moments
      if (cvplot == T){
        x11()
        if (moments==FALSE) op <- par(mfcol=c(2,1))
        else op <- par(mfcol=c(2,2))
      
        par(mfg=c(1,1))
    devcv <- rep(0,np)
        plot(index,moymean[index,j],pch=20,xlab="",ylab="Moyenne",ylim=c(min(moycimin[index,j]),max(moycimax[index,j])),main="")
# Petelet M.
#       polygon(c(index,index[length(index):1]),c(moycimax[index,j],moycimin[index[length(index):1],j]),col=gray(0.9),border = NA)
#       points(index,moymean[index,j],pch=21,xlab="",ylab="Moyenne",ylim=c(min(moycimin[index,j]),max(moycimax[index,j])),main="")
#       abline(h=mean(x[,j]),col="red")
        lines(index,moybias[index,j])
        lines(index,moycimin[index,j])
        lines(index,moycimax[index,j])
        lines(index,rep(moybias[n,j],nindex),col="blue",lty=3)
        lines(index,rep(moycimin[n,j],nindex),col="blue",lty=3)
        lines(index,rep(moycimax[n,j],nindex),col="blue",lty=3)
        if (moments==FALSE) title(c("Convergence des statistiques",nom[j]),sub="point : estimateur echantillon\n ligne : estimateur bootstrap + IC a 95%")
        else title("Convergence des statistiques",sub="point : estimateur echantillon")
        par(mfg=c(2,1))
        plot(index,ectmean[index,j],pch=20,xlab="taille de l'echantillon",ylab="Ecart-type",ylim=c(min(ectcimin[index,j]),max(ectcimax[index,j]))) 
        lines(index,ectbias[index,j])
        lines(index,ectcimin[index,j])
        lines(index,ectcimax[index,j])
        lines(index,rep(ectbias[n,j],nindex),col="blue",lty=3)
        lines(index,rep(ectcimin[n,j],nindex),col="blue",lty=3)
        lines(index,rep(ectcimax[n,j],nindex),col="blue",lty=3)

        if (moments==TRUE){
          par(mfg=c(1,2))
          plot(index,skewmean[index,j],pch=20,xlab="",ylab="Skewness",ylim=c(min(skewcimin[index,j]),max(skewcimax[index,j])),main="")
          lines(index,skewbias[index,j])
          lines(index,skewcimin[index,j])
          lines(index,skewcimax[index,j])
          lines(index,rep(skewbias[n,j],nindex),col="blue",lty=3)
          lines(index,rep(skewcimin[n,j],nindex),col="blue",lty=3)
          lines(index,rep(skewcimax[n,j],nindex),col="blue",lty=3)
          title(nom[j],sub="ligne : estimateur bootstrap + IC a 95%")
          par(mfg=c(2,2))
          plot(index,kurtmean[index,j],pch=20,xlab="taille de l'echantillon",ylab="Kurtosis",ylim=c(min(kurtcimin[index,j]),max(kurtcimax[index,j])))
          lines(index,kurtbias[index,j])
          lines(index,kurtcimin[index,j])
          lines(index,kurtcimax[index,j])
          lines(index,rep(kurtbias[n,j],nindex),col="blue",lty=3)
          lines(index,rep(kurtcimin[n,j],nindex),col="blue",lty=3)
          lines(index,rep(kurtcimax[n,j],nindex),col="blue",lty=3)
        }
#      par(op)
#      mtext("Convergence des statistiques",line=3)
#      mtext(dimnames(x)[[2]][j],line=1.5)
      }
#*****************************************************************
# convergence des accroissements des moments et des IC des moments

      if (jj==1){
        x11() # convergence des accroissements des moments 
        devcv[j] <- dev.cur()
        if (moments==FALSE) op <- par(mfcol=c(2,1))
        else op <- par(mfcol=c(2,2))
        par(mfg=c(1,1))
        plot(index,moyinc[index,j],type="l",xlab="",ylab="Moyenne",ylim=c(0,ampl*moyinc[index[length(index)],j]),main="")
        axis(1,tck=1,lty=2);axis(2,tck=1,lty=2)
        if (moments==F) title(nom[j],sub="Convergence de la somme des accroissements\n des moments statistiques")
        else title(nom[j],sub="               Convergence de la somme des")
        op1 <- par()
        par(mfg=c(2,1))
        plot(index,ectinc[index,j],type="l",xlab="taille de l'echantillon",ylab="Ecart-type",ylim=c(0,ampl*ectinc[index[length(index)],j]),main="")
        axis(1,tck=1,lty=2);axis(2,tck=1,lty=2)
        op2 <- par()
        if (moments==TRUE){
          par(mfg=c(1,2))
          plot(index,skewinc[index,j],type="l",xlab="",ylab="Skewness",ylim=c(0,ampl*skewinc[index[length(index)],j]),main="")
          axis(1,tck=1,lty=2);axis(2,tck=1,lty=2)
          title(nom[j],sub="accroissements des moments statistiques                  ")
          op3 <- par()
          par(mfg=c(2,2))
          plot(index,kurtinc[index,j],type="l",xlab="taille de l'echantillon",ylab="Kurtosis",ylim=c(0,ampl*kurtinc[index[length(index)],j]),main="")
          axis(1,tck=1,lty=2);axis(2,tck=1,lty=2)
          op4 <- par()
        }

        x11() # convergence des accroissements des IC des moments
        devcvIC[j] <- dev.cur()
        if (moments==FALSE) op <- par(mfcol=c(2,1))
        else op <- par(mfcol=c(2,2))

        par(mfg=c(1,1))
        plot(index,moyICinc[index,j],type="l",xlab="",ylab="Moyenne",ylim=c(0,ampl*moyICinc[index[length(index)],j]),main="")
        axis(1,tck=1,lty=2);axis(2,tck=1,lty=2)

        if (moments==F) title(nom[j],sub="Convergence de la somme des accroissements\n des IC des moments statistiques")
        else title(nom[j],sub="    Convergence de la somme des increments")
        opIC1 <- par()
        
        par(mfg=c(2,1))
        plot(index,ectICinc[index,j],type="l",xlab="taille de l'echantillon",ylab="Ecart-type",ylim=c(0,ampl*ectICinc[index[length(index)],j]),main="")
        axis(1,tck=1,lty=2);axis(2,tck=1,lty=2)
        opIC2 <- par()

        if (moments==TRUE){
          par(mfg=c(1,2))
          plot(index,skewICinc[index,j],type="l",xlab="",ylab="Skewness",ylim=c(0,ampl*skewICinc[index[length(index)],j]),main="")
          axis(1,tck=1,lty=2);axis(2,tck=1,lty=2)

          title(nom[j],sub="des IC des moments statistiques                        ")
          opIC3 <- par()

          par(mfg=c(2,2))
          plot(index,kurtICinc[index,j],type="l",xlab="taille de l'echantillon",ylab="Kurtosis",ylim=c(0,ampl*kurtICinc[index[length(index)],j]),main="")
          axis(1,tck=1,lty=2);axis(2,tck=1,lty=2)
          opIC4 <- par()
        }
      }
      else{
        dev.set(devcv[j]) # convergence des accroissements des moments
        par(op1)
        par(mfg=c(1,1))
        lines(index,moyinc[index,j])
        par(op2)
        par(mfg=c(2,1))
        lines(index,ectinc[index,j])
        if (moments==TRUE){
          par(op3)
          par(mfg=c(1,2))
          lines(index,skewinc[index,j])
          par(op4)
          par(mfg=c(2,2))
          lines(index,kurtinc[index,j])}

        dev.set(devcvIC[j]) # convergence des accroissements des IC des moments
        par(opIC1)
        par(mfg=c(1,1))
        lines(index,moyICinc[index,j])
        par(opIC2)
        par(mfg=c(2,1))
        lines(index,ectICinc[index,j])
        if (moments==TRUE){
          par(opIC3)
          par(mfg=c(1,2))
          lines(index,skewICinc[index,j])
          par(opIC4)
          par(mfg=c(2,2))
          lines(index,kurtICinc[index,j])}
      }
    }
  }
}
