
# *********************************************************************
# truncated.R : fonctions pour lois tronquees
#
#   Les lois tronquees normale, lognormale et gumbel sont definies avec 
#   des noms specifiques :
#          dtnorm   ptnorm   qtnorm   rtnorm
#          dtlnorm  ptlnorm  qtlnorm  rtlnorm
#          dtgumbel ptgumbel qtgumbel rtgumbel
#
#   Toute autre loi de R peut etre tronquee avec les fonctions  :
#          d.trunc.distr p.trunc.distr q.trunc.distr r.trunc.distr
#
#**********************************************************************
#      Auteurs : G. Pujol et B. Iooss
#**********************************************************************

library(evd) # Loi Gumbel

# *********************************************************************

# The Truncated Normal Distribution

# density
dtnorm <- function(x, mean = 0, sd = 1, min = -1e6, max = 1e6){
  out = dnorm(x, mean, sd) / (pnorm(max, mean, sd) - pnorm(min, mean, sd))
  out[(x < min) | (x > max)] = 0
  return(out)
}

# distribution function
ptnorm <- function(q, mean = 0, sd = 1, min = -1e6, max = 1e6){
  out = (pnorm(q, mean, sd) - pnorm(min, mean, sd)) /
    (pnorm(max, mean, sd) - pnorm(min, mean, sd))
  out[q < min] = 0
  out[q > max] = 1
  return(out)
}

# quantile function
qtnorm <- function(p, mean = 0, sd = 1, min = -1e6, max = 1e6){
  return(qnorm((1 - p) * pnorm(min, mean, sd) + p * pnorm(max, mean, sd),
               mean, sd))
}

# random generation
rtnorm <- function(n, mean = 0, sd = 1, min = -1e6, max = 1e6){
  return(qtnorm(runif(n), mean, sd, min, max))
}

#**************************************
# The Truncated Lognormal Distribution

# density
dtlnorm <- function(x, meanlog = 0, sdlog = 1, min = -1e6, max = 1e6){
  out = dlnorm(x, meanlog, sdlog) / (plnorm(max, meanlog, sdlog) - plnorm(min, meanlog, sdlog))
  out[(x < min) | (x > max)] = 0
  return(out)
}

# distribution function
ptlnorm <- function(q, meanlog = 0, sdlog = 1, min = -1e6, max = 1e6){
  out = (plnorm(q, meanlog, sdlog) - plnorm(min, meanlog, sdlog)) /
    (plnorm(max, meanlog, sdlog) - plnorm(min, meanlog, sdlog))
  out[q < min] = 0
  out[q > max] = 1
  return(out)
}

# quantile function
qtlnorm <- function(p, meanlog = 0, sdlog = 1, min = -1e6, max = 1e6){
  return(qlnorm((1 - p) * plnorm(min, meanlog, sdlog) + p * plnorm(max, meanlog, sdlog),
               meanlog, sdlog))
}

# random generation
rtlnorm <- function(n, meanlog = 0, sdlog = 1, min = -1e6, max = 1e6){
  return(qtlnorm(runif(n), meanlog, sdlog, min, max))
}

# ***************************************
# The Truncated Gumbel Distribution

# density
dtgumbel <- function(x, loc = 0, scale = 1, min = -1e6, max = 1e6){
  out = dgumbel(x, loc, scale) / (pgumbel(max, loc, scale) - pgumbel(min, loc, scale))
  out[(x < min) | (x > max)] = 0
  return(out)
}

# distribution function
ptgumbel <- function(q, loc = 0, scale = 1, min = -1e6, max = 1e6){
  out = (pgumbel(q, loc, scale) - pgumbel(min, loc, scale)) /
    (pgumbel(max, loc, scale) - pgumbel(min, loc, scale))
  out[q < min] = 0
  out[q > max] = 1
  return(out)
}

# quantile function
qtgumbel <- function(p, loc = 0, scale = 1, min = -1e6, max = 1e6){
  return(qgumbel((1 - p) * pgumbel(min, loc, scale) + p * pgumbel(max, loc, scale),
               loc, scale))
}

# random generation
rtgumbel <- function(n, loc = 0, scale = 1, min = -1e6, max = 1e6){
  return(qtgumbel(runif(n), loc, scale, min, max))
}

# ******************************************************
# lois tronquees generiques
# ******************************************************

## Deux modes :
## d.trunc.distr(x, distr = 'norm', trunc.int = c(-2, 2), mean = 0, sd = 1)
## d.trunc.distr(x, distr = c('dnorm', 'pnorm', 'qnorm', 'rnorm'), trunc.int = c(-2, 2), mean = 0, sd = 1)

d.trunc.distr <- function(x, distr, trunc.int, ...) {
  if (length(distr) == 1) {
    distr <- paste(c('d', 'p', 'q', 'r'), distr, sep = '')
  }
  p.min <- do.call(distr[2], list(trunc.int[1], ...))
  p.max <- do.call(distr[2], list(trunc.int[2], ...))
  d.x <- do.call(distr[1], list(x, ...))
  out <- d.x / (p.max - p.min)
  out[(x < trunc.int[1]) | (x > trunc.int[2])] <- 0
  return(out)
}

p.trunc.distr <- function(q, distr, trunc.int, ...) {
  if (length(distr) == 1) {
    distr <- paste(c('d', 'p', 'q', 'r'), distr, sep = '')
  }
  p.min <- do.call(distr[2], list(trunc.int[1], ...))
  p.max <- do.call(distr[2], list(trunc.int[2], ...))
  p.q <- do.call(distr[2], list(q, ...))
  out <- (p.q - p.min) / (p.max - p.min)
  out[q < trunc.int[1]] <- 0
  out[q > trunc.int[2]] <- 1
  return(out)
}

q.trunc.distr <- function(p, distr, trunc.int, ...) {
  if (length(distr) == 1) {
    distr <- paste(c('d', 'p', 'q', 'r'), distr, sep = '')
  }
  p.min <- do.call(distr[2], list(trunc.int[1], ...))
  p.max <- do.call(distr[2], list(trunc.int[2], ...))
  do.call(distr[3], list((1 - p) * p.min + p * p.max, ...))
}

r.trunc.distr <- function(n, distr, trunc.int, ...) {
  q.trunc.distr(runif(n), distr, trunc.int, ...)
}
